const path = require('path');

function babelFix(config) {
  if (config.parser !== 'babel-eslint') {
    return config;
  }

  // === Babel plugin (https://github.com/babel/eslint-plugin-babel)
  var babelPluginRules = [
    'new-cap',
    'camelcase',
    'no-invalid-this',
    'object-curly-spacing',
    'quotes',
    'semi',
    'no-unused-expressions',
    'valid-typeof',
  ];
  babelPluginRules.forEach(function(rule) {
    if (config.rules[rule]) {
      config.rules['babel/' + rule] = config.rules[rule];
      delete config.rules[rule];
    }
  });

  config.plugins.unshift('babel');

  return config;
}

module.exports = babelFix({
  root: true,
  // parser: 'babel-eslint',
  parserOptions: {
    ecmaVersion: 2022,
    sourceType: 'module',
    ecmaFeatures: {
      modules: true,
    },
  },
  reportUnusedDisableDirectives: true,
  extends: [
    'eslint:recommended',
    'plugin:import/recommended',
  ],
  rules: {
    // Possible Errors

    // 'for-direction': 'error',                       // in eslint:recommended
    // 'getter-return': 'error',                       // in eslint:recommended
    // 'no-async-promise-executor': 'error',           // in eslint:recommended
    'no-await-in-loop': 'warn',
    // 'no-compare-neg-zero': 'error',                 // in eslint:recommended
    // 'no-cond-assign': 'error',                      // in eslint:recommended
    'no-console': 'off',
    'no-constant-condition': ['error', {checkLoops: false}], // in eslint:recommended
    // 'no-control-regex': 'error',                    // in eslint:recommended
    // 'no-debugger': 'error',                         // in eslint:recommended
    // 'no-dupe-args': 'error',                        // in eslint:recommended
    // 'no-dupe-else-if': 'error',                     // in eslint:recommended
    // 'no-dupe-keys': 'error',                        // in eslint:recommended
    // 'no-duplicate-case': 'error',                   // in eslint:recommended
    // 'no-empty': 'error',                            // in eslint:recommended
    // 'no-empty-character-class': 'error',            // in eslint:recommended
    // 'no-ex-assign': 'error',                        // in eslint:recommended
    // 'no-extra-boolean-cast': 'error',               // in eslint:recommended
    'no-extra-parens': ['error', 'functions'],
    // 'no-extra-semi': 'error',                       // in eslint:recommended
    // 'no-func-assign': 'error',                      // in eslint:recommended
    // 'no-import-assign': 'error',                    // in eslint:recommended
    'no-inner-declarations': 'warn', // in eslint:recommended
    // 'no-invalid-regexp': 'error',                   // in eslint:recommended
    // 'no-irregular-whitespace': 'error',             // in eslint:recommended
    'no-loss-of-precision': 'error',
    // 'no-misleading-character-class': 'error',       // in eslint:recommended
    // 'no-obj-calls': 'error',                        // in eslint:recommended
    // 'no-prototype-builtins': 'error',               // in eslint:recommended
    // 'no-regex-spaces': 'error',                     // in eslint:recommended
    // 'no-setter-return': 'error',                    // in eslint:recommended
    // 'no-sparse-arrays': 'error',                    // in eslint:recommended
    'no-template-curly-in-string': 'off',
    // 'no-unexpected-multiline': 'error',             // in eslint:recommended
    // 'no-unreachable': 'error',                      // in eslint:recommended
    // 'no-unsafe-finally': 'error',                   // in eslint:recommended
    // 'no-unsafe-negation': 'error',                  // in eslint:recommended
    'no-useless-backreference': 'error',
    'require-atomic-updates': 'error',
    // 'use-isnan': 'error',                           // in eslint:recommended
    // 'valid-typeof': 'error',                        // in eslint:recommended

    // Best Practices

    'accessor-pairs': 'error',
    'array-callback-return': 'error',
    'block-scoped-var': 'error',
    'class-methods-use-this': 'off',
    complexity: 'off',
    'consistent-return': 'error',
    curly: 'error',
    'default-case': 'error',
    'default-case-last': 'off',
    'default-param-last': 'error',
    'dot-location': ['error', 'property'],
    'dot-notation': ['error', {allowKeywords: true}],
    eqeqeq: ['error', 'smart'],
    'grouped-accessor-pairs': ['error', 'getBeforeSet'],
    'guard-for-in': 'error',
    'max-classes-per-file': 'off',
    'no-alert': 'warn',
    'no-caller': 'error',
    // 'no-case-declarations': 'error',                // in eslint:recommended
    'no-constructor-return': 'error',
    'no-div-regex': 'off',
    'no-else-return': 'off',
    'no-empty-function': 'error',
    // 'no-empty-pattern': 'error',                    // in eslint:recommended
    'no-eq-null': 'off', // handled by 'eqeqeq'
    'no-eval': 'error',
    'no-extend-native': 'error',
    'no-extra-bind': 'error',
    'no-extra-label': 'error',
    // 'no-fallthrough': 'error',                      // in eslint:recommended
    'no-floating-decimal': 'off',
    // 'no-global-assign': 'error',                    // in eslint:recommended
    'no-implicit-coercion': ['error', {allow: ['!!']}],
    'no-implicit-globals': 'error',
    'no-implied-eval': 'error',
    // 'no-invalid-this': 'error',
    'no-iterator': 'error',
    'no-labels': 'error',
    'no-lone-blocks': 'error',
    'no-loop-func': 'error',
    // 'no-magic-numbers': ['error', {ignore: [0, 1, 2]}],
    'no-multi-spaces': 'off',
    'no-multi-str': 'error',
    'no-new': 'error',
    'no-new-func': 'error',
    'no-new-wrappers': 'error',
    // 'no-octal': 'error',                            // in eslint:recommended
    'no-octal-escape': 'error',
    'no-param-reassign': 'off',
    'no-proto': 'error',
    // 'no-redeclare': 'error',                        // in eslint:recommended
    'no-restricted-properties': 'off',
    'no-return-assign': 'error',
    'no-return-await': 'error',
    'no-script-url': 'error',
    // 'no-self-assign': 'error',                      // in eslint:recommended
    'no-self-compare': 'error',
    'no-sequences': 'error',
    'no-throw-literal': 'error',
    'no-unmodified-loop-condition': 'error',
    'no-unused-expressions': 'error',
    // 'no-unused-labels': 'error',                    // in eslint:recommended
    'no-useless-call': 'error',
    // 'no-useless-catch': 'error',                    // in eslint:recommended
    'no-useless-concat': 'error',
    // 'no-useless-escape': 'error',                   // in eslint:recommended
    'no-useless-return': 'error',
    'no-void': 'error',
    'no-warning-comments': 'warn',
    // 'no-with': 'error',                             // in eslint:recommended
    'prefer-named-capture-group': 'off',
    'prefer-promise-reject-errors': 'error',
    'prefer-regex-literals': 'error',
    radix: 'error',
    'require-await': 'error',
    // 'require-unicode-regexp': 'warn',
    // 'vars-on-top': 'warn',
    'wrap-iife': ['error', 'inside'],
    // yoda: ['error', 'never', {exceptRange: true}],

    // Strict Mode

    strict: 'error',

    // Variables

    'init-declarations': 'off',
    // 'no-delete-var': 'error',                       // in eslint:recommended
    'no-label-var': 'error',
    'no-restricted-globals': 'off',
    'no-shadow': 'warn',
    // 'no-shadow-restricted-names': 'error',          // in eslint:recommended
    // 'no-undef': 'error',                            // in eslint:recommended
    'no-undef-init': 'error',
    'no-undefined': 'off',
    'no-unused-vars': ['error', {args: 'none'}], // in eslint:recommended
    'no-use-before-define': ['error', 'nofunc'],

    // Stylistic Issues

    'array-bracket-newline': ['error', 'consistent'],
    'array-bracket-spacing': 'error',
    'array-element-newline': 'off',
    'block-spacing': 'error',
    'brace-style': ['off', '1tbs'], // or 'allman'
    camelcase: ['warn', {properties: 'never'}],
    'capitalized-comments': 'off',
    'comma-dangle': ['error', 'only-multiline'],
    'comma-spacing': ['error', {before: false, after: true}],
    'comma-style': ['error', 'last'],
    'computed-property-spacing': ['error', 'never'],
    'consistent-this': 'off',
    'eol-last': ['error', 'always'],
    'func-call-spacing': 'error',
    'func-name-matching': 'off',
    'func-names': 'off',
    'func-style': ['error', 'declaration', {allowArrowFunctions: false}],
    'function-call-argument-newline': 'off',
    'function-paren-newline': 'off',
    'id-blacklist': 'off',
    'id-length': 'off',
    'id-match': 'off',
    'implicit-arrow-linebreak': ['off', 'beside'],
    indent: [
      'error',
      2,
      {
        VariableDeclarator: {var: 2, let: 2, const: 3},
        MemberExpression: 'off',
        ignoredNodes: [
          'VariableDeclarator > ConditionalExpression',
          'Property > ConditionalExpression',
          ':expression > ConditionalExpression',
          ':statement > ConditionalExpression',
        ],
      },
    ],
    'jsx-quotes': ['error', 'prefer-double'],
    // 'key-spacing': [
    //   'warn',
    //   {
    //     singleLine: {
    //       beforeColon: false,
    //       afterColon: true,
    //     },
    //     multiLine: {
    //       align: 'colon',
    //     },
    //   },
    // ],
    'keyword-spacing': ['error', {before: true, after: true}],
    'line-comment-position': 'off',
    'linebreak-style': ['error', 'unix'],
    'lines-around-comment': 'off',
    'lines-between-class-members': 'off',
    'max-depth': ['warn', 5],
    'max-len': [
      'error',
      {
        code: 120,
        tabWidth: 2,
        ignorePattern: '^(?:RegExp\\(.*?\\))|(?:/.*?/.*)$',
        ignoreComments: false,
        ignoreTrailingComments: true,
        ignoreTemplateLiterals: true,
        ignoreUrls: true,
      },
    ],
    'max-lines': [
      'error',
      {
        max: 1000,
        skipBlankLines: true,
        skipComments: true,
      },
    ],
    'max-lines-per-function': 'off',
    'max-nested-callbacks': ['error', 5],
    'max-params': ['off', 6],
    'max-statements': 'off',
    'max-statements-per-line': ['error', {max: 1}],
    'multiline-comment-style': 'off',
    'multiline-ternary': 'off',
    'new-cap': [
      'error',
      {
        capIsNewExceptions: ['DOMEvents'],
        // Only if the name contains a lower case letter
        capIsNewExceptionPattern: '^[^a-z]+$',
      },
    ],
    'new-parens': 'error',
    'newline-per-chained-call': 'off',
    'no-array-constructor': 'error',
    'no-bitwise': ['error', {int32Hint: true}],
    'no-continue': 'off',
    'no-inline-comments': 'off',
    'no-lonely-if': 'error',
    // 'no-mixed-operators': 'warn', // incopatible with prettier
    // 'no-mixed-spaces-and-tabs': 'error',            // in eslint:recommended
    // 'no-multi-assign': 'error',
    'no-multiple-empty-lines': 'off',
    'no-negated-condition': 'error',
    'no-nested-ternary': 'off',
    'no-new-object': 'error',
    'no-plusplus': 'off',
    'no-restricted-syntax': ['error', 'WithStatement'],
    'no-tabs': 'error',
    'no-ternary': 'off',
    'no-trailing-spaces': 'error',
    'no-underscore-dangle': 'off',
    'no-unneeded-ternary': 'error',
    'no-whitespace-before-property': 'error',
    'nonblock-statement-body-position': 'off',
    'object-curly-newline': ['error', {consistent: true}],
    'object-curly-spacing': ['error', 'never'],
    'object-property-newline': 'off',
    'one-var': ['error', 'never'],
    'one-var-declaration-per-line': ['error', 'initializations'],
    'operator-assignment': ['error', 'always'],
    'operator-linebreak': [
      'off',
      'after',
      {overrides: {'?': 'before', ':': 'before'}},
    ],
    'padded-blocks': 'off',
    'padding-line-between-statements': [
      'error',
      {blankLine: 'always', prev: 'directive', next: '*'},
      {blankLine: 'any', prev: 'directive', next: 'directive'},

      {blankLine: 'always', prev: 'import', next: '*'},
      {blankLine: 'any', prev: 'import', next: 'import'},

      // {blankLine: 'always', prev: ['const', 'let', 'var'], next: '*'},
      // {
      //   blankLine: 'any',
      //   prev: ['const', 'let', 'var'],
      //   next: ['const', 'let', 'var'],
      // },

      {blankLine: 'always', prev: '*', next: 'class'},
    ],
    'prefer-exponentiation-operator': 'off',
    'prefer-object-spread': 'off',
    'quote-props': ['error', 'as-needed', {numbers: true}],
    quotes: [
      'error',
      'single',
      {avoidEscape: true, allowTemplateLiterals: false},
    ],
    semi: ['error', 'always'],
    'semi-spacing': ['error', {before: false, after: true}],
    'semi-style': ['error', 'last'],
    'sort-keys': 'off',
    'sort-vars': 'off',
    'space-before-blocks': ['error', 'always'],
    'space-before-function-paren': 'off',
    'space-in-parens': ['error', 'never'],
    'space-infix-ops': ['error', {int32Hint: true}],
    'space-unary-ops': ['error', {words: true, nonwords: false}],
    'spaced-comment': 'off',
    'switch-colon-spacing': ['error', {before: false, after: true}],
    'template-tag-spacing': ['error', 'never'],
    'unicode-bom': ['error', 'never'],
    'wrap-regex': 'off',

    // ECMAScript 6

    'arrow-body-style': ['error', 'as-needed'],
    'arrow-parens': ['error', 'always'],
    'arrow-spacing': 'error',
    // 'constructor-super': 'error',                   // in eslint:recommended
    'generator-star-spacing': [
      'error',
      {
        before: false,
        after: true,
        anonymous: 'neither',
        method: {before: true, after: false},
      },
    ],
    // 'no-class-assign': 'error',                     // in eslint:recommended
    'no-confusing-arrow': ['error', {allowParens: true}],
    // 'no-const-assign': 'error',                     // in eslint:recommended
    // 'no-dupe-class-members': 'error',               // in eslint:recommended
    // 'no-duplicate-imports': 'error', // use 'import/no-duplicates' instead
    // 'no-new-symbol': 'error',                       // in eslint:recommended
    'no-restricted-exports': 'off',
    'no-restricted-imports': 'off',
    // 'no-this-before-super': 'error',                // in eslint:recommended
    'no-useless-computed-key': 'error',
    'no-useless-constructor': 'error',
    'no-useless-rename': 'off',
    'no-var': 'off',
    'object-shorthand': 'off',
    'prefer-arrow-callback': 'off',
    'prefer-const': 'off',
    'prefer-destructuring': 'off',
    'prefer-numeric-literals': 'error',
    'prefer-rest-params': 'off',
    'prefer-spread': 'off',
    'prefer-template': 'off',
    // 'require-yield': 'error',                       // in eslint:recommended
    'rest-spread-spacing': ['error', 'never'],
    'sort-imports': 'off',
    'symbol-description': 'warn',
    'template-curly-spacing': ['error', 'never'],
    'yield-star-spacing': ['error', {before: false, after: true}],

    // === Compat plugin (https://github.com/amilajack/eslint-plugin-compat)

    'compat/compat': 'warn',

    // === Import plugin (https://github.com/benmosher/eslint-plugin-import)
    // Static analysis

    'import/no-unresolved': 'off', // in plugin:import/recommended
    // 'import/named': 'error',                      // in plugin:import/recommended
    // 'import/default': 'error',                    // in plugin:import/recommended
    // 'import/namespace': 'error',                  // in plugin:import/recommended
    'import/no-restricted-paths': 'off',
    'import/no-absolute-path': 'off',
    'import/no-dynamic-require': 'error',
    'import/no-internal-modules': 'off',
    'import/no-webpack-loader-syntax': 'off',
    'import/no-self-import': 'error',
    'import/no-cycle': 'warn',
    'import/no-useless-path-segments': 'warn',
    'import/no-relative-parent-imports': 'off',

    // Helpful warnings

    // 'import/export': 'error',                   // in plugin:import/recommended
    // 'import/no-named-as-default': 'warn',       // in plugin:import/recommended
    // 'import/no-named-as-default-member': 'warn',// in plugin:import/recommended
    'import/no-deprecated': 'warn',
    'import/no-extraneous-dependencies': 'warn',
    'import/no-mutable-exports': 'off',
    'import/no-unused-modules': 'off',

    // Module systems

    'import/unambiguous': 'off',
    'import/no-commonjs': 'error',
    'import/no-amd': 'error',
    'import/no-nodejs-modules': 'error',

    // Style guide

    'import/first': 'off',
    'import/exports-last': 'off',
    'import/no-duplicates': 'error',             // in plugin:import/recommended
    'import/no-namespace': 'off',
    'import/extensions': ['error', 'ignorePackages'],
    'import/order': 'warn',
    'import/newline-after-import': 'error',
    'import/prefer-default-export': 'off',
    'import/max-dependencies': 'off',
    'import/no-unassigned-import': 'off',
    'import/no-named-default': 'off',
    'import/no-default-export': 'warn',
    'import/no-named-export': 'off',
    'import/no-anonymous-default-export': 'off',
    'import/group-exports': 'off',
    'import/dynamic-import-chunkname': 'off',
  },
  plugins: ['compat', 'import'],
  settings: {
    polyfills: [
      'Object.fromEntries',
      'Object.hasOwn',
      'Promise.allSettled',
      'AbortSignal',
      'AbortController',
      'CSS.escape',
    ],
  },
  env: {
    browser: true,
    es6: true,
  },
  globals: {
    "ODL": true,
  },
});
