# Changelog

## 12-07-2024

### 🔨 Improvements

* Improve consistency of scroll-based interaction across platforms and input
  devices (mice and trackpads).
* Vary scroll speed on the coverage timeline with the selected timespan.

### 🧑‍💻 For Developers

* All scroll-based interactions now use `Gestures.getWheelHandler()`, for which
  the API has changed and no-longer supports passing a sensitivity option.

---

## 08-07-2024

### 🐛 Bug fixes

* Fix detailed timeline dragging issues on iOS's and iPadOS's Safari.

---

## 25-06-2024

### 🎉 New features

* Add support for zooming into raster layers beyond their maximum available zoom
  level.
* Add support for placing vector products (like trajectories) under raster ones.

### 🔨 Improvements

* Use NS/EW coordinate notation in the map grid
* Add outline around text to improve its readability.
* Flip coordinates order showing the latitude then the longitude.
* Dynamically adapt the layout of the timeline calendar on small screens.
* Explain the coloc colors in the tooltips of the toggle button and the years,
  months and days in timeline.
* Add support timespans with multiple components (ex. `1d30m`).
* Add support for fine-grain scrolling in the details timeline (ex. when
  scrolling using a touchpad).
* Insure that the month/year boundary is always shown in the details timeline.
* Activate colocation highlighting in the timeline by default.
* Highlight the current date/time and timespan in all rows of the coverage
  timeline. Small highlights will be hidden.

### 🐛 Bug fixes

* Fix unintended "Unhandled promise rejection" errors.
* Fix unintended errors when persistent storage is blocked and the user:
  * selects a dataset, or
  * opens the "What's new!" dialog.
* Fix misplaced streamlines in projections without IDL wrapping.
* Fix bug where dataset's attached image opens small in Safari.
* Fix go to nearest when used in combination with the coloc highlighting feature.
* Fix days in the timeline calendar overlapping the iOS/iPadOS's home bar at the
  bottom of the screen.

### 🧑‍💻 For Developers

* Update `esbuild` to `v0.21.5`.
* Update `marked` to `v13.0.0`.
* Add a type checker to mark its arg as required.
* Add support for accessing a dataset's raw metadata.
* Add support for comparator in `addToSorted` utils.
* Prevent swallowing errors in throttled functions.
* Augment unit conversion with a new `Quantity` class representing a possibly
  imprecise numeric value with an optional unit. With support for unit
  conversion and formatting.
* Add support for debug build detection at runtime using `window.DEBUG`.
* Colocation highlighting is configurable using the `colocActive` config field.

---

## 15-05-2024

### 🐛 Bug fixes

* Avoid mutating GeoJSON objects during parsing

---

## 17-04-2024

### 🐛 Bug fixes

* Fix race-condition causing an inconsistent stack order of datasets

---

## 13-03-2024

### 🐛 Bug fixes

* Fix an unhandled promise rejection alert when in-browser storage is unavailable

### 🔨 Improvements

* Optimize color and color palette modules
* Optimize streamlines' memory footprint

### 🧑‍💻 For Developers

* Update Google Maps version to v3.56
* Improve config loading error message
* Allow easy troubleshooting of config errors
* Add support for a type checker requiring any value
* Add function to assert the validity of a value
* Replace config validators with assertion functions

---

## 06-03-2024

### 🐛 Bug fixes

* Fix selecting text annotations

---

## 14-02-2024

### 🐛 Bug fixes

* Fix error when clicking the "Share" button with "Permalink" dialog is visible
* Fix computed metrics of annotations given in "Selection info" panel
* Fix importing GeoJSON that don't have the non-standard "crs" property

---

## 03-01-2024

### 🔨 Improvements

* Several adjustments to improve the usability of the detailed timeline,
  accessed by clicking the upward-pointing triangle located directly above the
  timeline, notably the increase in lines thickness and the addition of more
  time markers.
* Avoid shifting Display data's content when using <kbd>shift</kbd>+<kbd>F</kbd>
* Replace "Request Feature" with "Suggest Feature" in the "Feedback" menu
* Change color of the remove product button in "Display data" panel

### 🧑‍💻 For Developers

* Specify a canonical URL for search engines
* Set the title bar's color scheme to a dark
* Add support specifying style of title bar buttons and their indicators
* Fix log unit parsing in "Display data"
* Fix edge case when computing geodesic distance
* Move OL's layer tiles access patches into core
* Update marked to v11.1.0
* Update esbuild to v0.19.9
* Update Font Awesome to v4.7.0

---

## 27-11-2023

### 🎉 New features

* Replace the "Feedback" dialog with a title bar menu with option to send an email

* <details><summary>Add product hidden indicator in the "Products" side panel</summary>

    A slashed eye icon will replace the checkmark next to a selected product in
    the "Products" side panel, indicating that the product is currently hidden.
    Placing the mouse pointer over that icon will show a tooltip with list of
    required steps to see the product.

    </details>

* <details><summary>Add button to remove a product from the "Display data" panel</summary>

    Each product in the "Display data" panel has a red cross button located at
    the far right of its row. That button can be used to remove its
    corresponding product from the list.
    A product removed from the "Display data" panel can always be added back by
    selecting it in the "Products" side panel.

    </details>

### 🔨 Improvements

* Dim the text on deselected products in the "Display data" panel
* Replace the icons of the import and export buttons in the title bar, to avoid
  confusions reported by some users

### 🐛 Bug fixes

* Fix datasets filters active indicator not shown after initial load
* Fix products list not updated when server reachability changes

---

## 03-11-2023

### 🎉 New features

* Add button to clear a custom filter field in the "Display data" panel
* Add an "About" dialog which is shown on first visit and can be subsequently
  accessed via the "Help" button in the title bar

* <details><summary>Add a datasets filters active indicator on the "Display data" button in the title bar</summary>

    An exclamation mark will showup on the "Display data" button in the title
    bar to indicate that some data may be hidden by the applied filters.

    </details>

* <details><summary>Add button for quickly isolating the selected dataset</summary>

    The "Filter using ID" button, in the "Selection info" panel, appears when
    placing the mouse cursor over the ID row and toggles the filtering of the
    selected dataset's product using the ID of said dataset, thus providing a
    fast way to isolate a single dataset of a given product.

    </details>

### 🔨 Improvements

* Improve readability in "Display data" panel by changing the predefined
  filters' checkboxes color and increasing their left-side spacing
* Dim the colors on unselected products in the "Display data" panel
* Show the label of a field when copying its value in the "Selection info" panel
* Update the title bar's button grouping
* Remove obsolete contents and modernize the guided tour

### 🐛 Bug fixes

* Fix behavior of arrow keys after clicking on timespan so they change the
  current datetime instead of switching to another timespan

### 🧑‍💻 For Developers

* Added "powered by Syntool" under the title of the portal
* The following Syntool methods will now abort the previous call to avoid race
  conditions:

  * updateCoverage()
  * loadDatasetInfos()
  * gotoNearest()

---

## 13-10-2023

### 🎉 New features

* <details><summary>Easy filtering of a product's datasets using predefined filters</summary>

    A product's predefined filters are shown in the "Display data" panel under
    its label. Giving a fast and easy way to filter a product's datasets. Custom
    filters are still available through the funnel toggle to the right of the
    opacity gauge control.

    When selecting datasets on the map, the filters that match those datasets
    will get highlighted in orange.

    </details>

### 🔨 Improvements

* Replace time in URL with a readable date
* Avoid excessive %-encoding in browser's the address bar

### 🐛 Bug fixes

* Fix bug wherein the geographical filtering of the datasets and coverage are
not updated when switching base layers
* Fix bug wherein an error occurs in certain cases when loading Syntool

### 🧑‍💻 For Developers

* Show deprecated config server options at the end of the validation suggestions

---

## 11-10-2023

### 🐛 Bug fixes

* Fix error when loading some trajectories with invalid GeoJSON by relaxing the
  requirement that a "LineString" must have at least 2 points
* Fix bug where trajectories IDL wrap correction was not applied when
  `wrapDateLine` is set to `false` on the base layer. This correction is
  required for old datasets that were created with a bad wrapping algorithm

---

## 05-10-2023

### 🐛 Bug fixes

* Fix rare bug causing some datasets to be missing when loading from API cache
* Fix bugs related to handling of `wrapDateLine` set to `false` on the base
  layer

### 🧑‍💻 For Developers

* Add JSDoc with TypeScript type annotations in core

---

## 15-09-2023

### 🎉 New features

* <details><summary>New keyboard shortcuts for some drawing tools</summary>

    * <kbd>D</kbd> for a point
    * <kbd>L</kbd> for a polyline
    * <kbd>P</kbd> for a polygon

    </details>

* <details><summary>Always enabled continuous drawing for polylines and polygons</summary>

    Continuous drawing allows placing points like before by clicking or taping,
    and also drawing lines by dragging like when using a pen.

    To pan the map while drawing you can use the middle mouse button, hold
    <kbd>Ctrl</kbd> (or <kbd>⌘ Command</kbd> on macOS), or use two-finger
    gestures on touch enabled devices.

    </details>

* <details><summary>Add undo/redo while drawing a polyline or a polygon</summary>

    Pressing the "Undo" button (or its keyboard shortcut
    <kbd>Ctrl</kbd>+<kbd>Z</kbd> or <kbd>⌘ Command</kbd>+<kbd>Z</kbd>) while
    drawing a polyline or a polygon will remove the last drawn segment.

    Similarly pressing the "Redo" button (or its keyboard shortcut
    <kbd>Ctrl</kbd>+<kbd>Shift</kbd>+<kbd>Z</kbd> or
    <kbd>⌘ Command</kbd>+<kbd>Shift</kbd>+<kbd>Z</kbd>) will redraw the last
    undone segment.

    </details>

* <details><summary>Add snapping to control points while drawing</summary>

    When an annotation is selected, its control points are shown. Starting a
    drawing operation while in that state activates snapping to the shown
    control points.

    </details>

* <details><summary>Add support for extending a polyline</summary>

    Extending a polyline can be done by first selecting said polyline to show
    its points. Then activating the polyline drawing tool and drawing the
    extension starting from the extremity that needs to be extended.

    </details>

* <details><summary>Quick drawing for arrows and eddies/ellipses annotations</summary>

    Enable quick drawing of an arrow annotation by pressing and holding the left
    mouse button where the arrow should start, moving the mouse to where the
    arrow should be pointing, then releasing the mouse button. Or by using a
    similar gesture on touch enabled devices.

    Likewise enable quick placing and resizing of an eddy/ellipse using the same
    gesture.

    </details>

* <details><summary>Allow moving the jet of a hammerhead/mushroom annotation</summary>

    Add a control point that controls the position of the inner extremity of the
    annotations' jet. Giving the flexibility to better annotate
    hammerheads/mushrooms.

    </details>

* Add option "None" to mark arrows on a polyline or an eddy/ellipse as having an
  unspecified "Hot" side (which became the default for polylines)

* Add support for removing annotation vertices by clicking/tapping them

### 🔨 Improvements

* Major updates to the annotation toolbar
  * Slicker design
  * Updated icons
  * New undo and redo buttons
  * New "Done" button when drawing
  * New buttons to duplicate or delete selected annotation(s)
* Move the rotate/scale control point of annotations out of the way of other
  control points
* Limit the minimum scaling of an annotation to a few pixels on the screen
* Increase the hit width of polyline annotation
* Hide intermediate midpoints (allowing to add vertices) of an annotation when
  there isn't enough space, thus allowing the user to interact with the other
  vertices (which the midpoint prevents)

### 🐛 Bug fixes

* Open "Selection info" panel when a text field within it receives focus
* Prevent macOS's data detector from triggering unexpectedly when editing an
  annotation

### 🗑️ Feature removals

* Remove double-click/double-tap to finish drawing, use the "Done" button
  or the <kbd>Enter</kbd> (or <kbd>Return</kbd>) key instead

---

## 18-08-2023

### 🎉 New features

* Add button to clear the catalog search field
* Allow map to cross the international date line so that it is possible to
  center the view on the Pacific ocean
* Add support for collapsing blocks in the "Selection info" panel
* Avoid reopening the "Selection info" panel after being closed by the user
* The button for opening/closing the "Selection info" panel now changes color
  and bounces when new information is available while the panel is closed

### 🔨 Improvements

* New optimized rendering engine for vector features (ie. trajectories, markers,
  etc...)
* Improve Google Maps integration
* Improve memory management
* Improve inertial map panning
* Improve trajectories markers
* Improve drag'n'drop reordering in "Display data"
* Drawing tools are now always visible, even when the "User Shapes" product is
  not selected
* Permanent datasets (including user shapes) are ignored when computing data
  availability
* Shortcut for deleting a user shape is now <kbd>Delete</kbd> instead of
  <kbd>Shift</kbd>+<kbd>Delete</kbd>
* Prefer multiple of 45° for lat/lon grid ticks when possible
* Change mouse cursor to grabbing when panning map
* Prevent state loss when page is reloaded or unloaded by the browser
* Ensure a consistent datasets ordering on the map
* Allow the processing tools button to remain enabled

### 🐛 Bug fixes

* Workaround bug in Firefox Focus preventing Syntool from loading
* Fix text color of info panel buttons on iOS Safari
* Fix many bugs when handling geometries and interactions that cross the
international date line
* Fix initial automatic jump to nearest date with data
* Fix bug wherein catalog filter was ignored after collapsing a group
* Fix bug when zooming while panning the map
* Fix marking the "Text" field of an annotation erroneously invalid
* Fix showing the drop zone on Safari
* Fix importing polygon with holes from KML
* Fix rendering bug for polygons with holes
* Fix vertical misalignment of shapes text on Chrome
* Fix bug wherein the "Selection info" panel contents are erroneously cutoff
* Prevent emptying the "Text" field of a text annotations

### 🧑‍💻 For Developers

* Drop support for legacy Edge browser and all browsers not supporting dynamic
  imports
* Remove jQuery and proj4js dependencies
* Replace custom base map picker with a `<select>`
* Add attribution control
* Reduce the default `animationSpeed` parameter 8x and add an 8x animation speed
  option in the timeline
* Limit the number of active AJAX requests to avoid timeout errors caused by the
web browser mechanism for queueing requests
* Fix precision bug when computing geometry's centroid
* Double transform matrix calculations precision
* Add global error handlers that notify the user
* Improve "out of memory" error detection
* Clear API cache as last resort when memory is low
* Remove old datasets when memory is low
* Retry initializing streamlines renderer in low memory
* Add support for resolution dependent layer styles
* Add support for custom node based marker symbols
* Add support for returning `null` from a `MarkerFn`
* Add support for excluding products from coloc
* Add support for `graphicOpacity` marker property
* Add support for using `await` in the `config.js` file
* Add support for importing shapes into a specific product
* Add support for importing shapes from linked URLs into separate products
* Add support for removable products
* Add support for having multiple versions of the `index.html`
* Add support for scaling the Syntool instance
* Add support for monitoring loading of Google Maps tiles
* Add support for hover events on title bar buttons
* Add Syntool wide state management
* Add support for Syntool controls and refactor to UI to use them
* Simplify extension mechanism of `Syntool` class

---

## 07-02-2022

### 🎉 New features

* Add support linking a product with multiple server products (even across
multiple servers).
* Add support for configuring the streamlines speed factor of a product.

### 🔨 Improvements

* Improve support for trackpad/touchpad scrolling
* Add support for wrapping IMAGE datasets when they cross the international
  date line

### 🐛 Bug fixes

* <details><summary>Fix bug preventing streamlines from crossing the international date line</summary>

    When viewing the wrap of a streamlines dataset that crosses the IDL,
    it appears mostly empty.
    This was caused by a miscalculation when computing the density factor,
    using the wrong width of the streamlines area intersecting the viewport,
    the density factor was set to NaN implying no streamlines particles.

    </details>

* <details><summary>Fix bug causing some markers to be in the wrong position on older version of Safari</summary>

    Safari did not support the SVG "transform-origin" attribute until probably
    version 15.

    </details>

---

## 31-01-2022

### 🎉 New features

* Add support linking a product with multiple server products (even across
multiple servers).

### 🐛 Bug fixes

* Fix typo causing an error when parsing GeoJSON in a different projection.

---

## 12-01-2022

### 🔨 Improvements

* Include a warning about modern web browsers lacking support for FTP in
  tooltips related to FTP links.

---

## 23-12-2021

### 🎉 New features

* Add product's ID as tooltip in "Display data" panel.

### 🔨 Improvements

* Add support for extending Syntool's state
* Add support for extending the application's state
* Add support for creating short links with catalog selection
* Add support for handling truncated API responses
* Add support for creating stepped color palette from colors
* Fix computing the cached datasets time range

### 🧹 Chore

* Replace JSONP `/datasets` request with JSON
* Improve color palette logging in the console
* Simplify color palette creation and warn on CPT parse errors

---

## 14-12-2021

### 🐛 Bug fixes

* Fix minor bug wherein the streamlines blinked at the wrong position after
panning the map

### 🎉 New features

* Add product's ID as tooltip in "Display data" panel.

### 🔨 Improvements

* Add support for extending Syntool's state
* Add support for extending the application's state
* Add support for creating short links with catalog selection

---

## 03-11-2021

### 🐛 Bug fixes

* Fix bug wherein some trajectory markers' position seam blocked.

### 🔨 Improvements

* Update the browser detection logic.
* Replace using `innerHTML` with the cleaner `DOMParser` for transforming HTML
  text.

### 🧹 Chore

* Bump the minimum supported version of Edge to 17.
* Reverse the insertion order of `addCSSRuleToSheet()`.

---

## 27-09-2021

### 🐛 Bug fixes

* Fix a bug caused by a race condition affecting the visibility of NCWMS and
TIMELAYER product types.

---

## 15-09-2021

### 🎉 New features

* Add support for KML and GeoJSON formats when importing and exporting shape
annotations.

### 🔨 Improvements

* Revamp the shape annotation's context menu to more compact and accessible.
This menu is shown on the right when one or more annotations are selected.

---

## 07-07-2021

### 🐛 Bug fixes

* Fix the value of the "Area (NM²)" metric.
* Fix a rare case where the "copy to clipboard" didn't work.
* Fix deleting a user shape vertex on macOS using the <kbd>⌫</kbd> (backspace).

### 🔨 Improvements

* Use macOS shortcuts, instead of PC ones, on all Apple devices like iPads
(with a keyboard connected).

### 🎉 New features

* <details><summary>Add buttons to copy the value of any field in the "Selection info" panel</summary>

    When hovering over a row in the "Selection info" panel, a "copy to
    clipboard" button will appear on the right.

    Clicking it will copy the value of that row to the clipboard.

    </details>

* <details><summary>Add a continuous drawing mode for polylines and polygons</summary>

    When drawing a polyline or a polygon, you can hold down <kbd>Ctrl</kbd>
    (or <kbd>Cmd ⌘</kbd> on Apple devices) to activate the continuous drawing mode.

    In that mode, you can use your mouse/finger to draw freely (as opposed to
    placing points one by one), and the map will not pan.

    Releasing the activation key will deactivate that mode.

    </details>

---

## 02-06-2021

### 🔨 Improvements

* Avoid layout shifts in the timeline bar when showing/hiding the coordinates
crosshairs icon.

---

## 26-05-2021

### 🐛 Bug fixes

* Fix bug causing some external products (like "Natural Color Meteosat 10
(Eumetsat)") to have datasets without an ID.

---

## 11-05-2021

### 🐛 Bug fixes

* Fix bug causing an error when a product is added dynamically, this had
probably no effect for users.

### 🔨 Improvements

* <details><summary>Avoid reordering products in the catalog based on the filter's match position</summary>

    When searching in the products catalog, the results are ordered by
    relevance. Which is determined based on how well the product matches the
    search terms and the position of that match.

    This change avoid the position of the match when determining the relevance
    of a product.

    </details>

---

## 06-05-2021

### 🐛 Bug fixes

* Fix bug in the configuration validator which caused it to ignore parts of
the configuration.
* Fix bugs that were undetected before fixing the validator, they had no
apparent effects for users.
* <details><summary>Fix bug causing some permanent datasets to not be visible in the detailed timeline</summary>

    Datasets that are not limited in time are considered permanent (eg.
    `mdt_cnes_cls2013_global` dataset of the `3857_Mean_Dynamic_Topography`
    product).
    When rendering the line segment representing a such dataset in the detailed
    timeline, the former ends up having a width greater than the maximum allowed
    by the browser which treats it as if was the maximum.
    This in turn prevents the line segment from being visible in some parts of
    the detailed timeline.
    This case has been observed when using Firefox v89.

    This has been fixed by clipping the line segments when rendering to a
    sub-portion of the timeline containing the visible portion.

    </details>

* <details><summary>Fix bug causing invalid requests being sent to the API servers in some cases</summary>

    This bug is due to a miss-implemented utility function `timeToTimestamp()`
    that converts it's input in milliseconds to integer seconds (rounding down).
    That function used the bitwise or (`|`) operator to convert to integer.
    But that operator returns a signed 32-bit integer, which is a problem when
    the input number is greater than `2^31-1`.

    The solution was to replace the bitwise or (`|`) with a unsigned right
    shift operator (`>>>`), which returns an unsigned 32-bit integer.

    </details>

* <details><summary>Fix auto jump-to-nearest when selecting the first product</summary>

    When no products are selected and you select a single one, you expect the
    current date-time to change to one with data (ie. an automatic go-to-nearest
    to be triggered).
    But it wasn't (it actually was but was subsequently canceled shortly after).
    The go-to-nearest request was being canceled when Syntool's "update" event
    was emitted (which happens shortly after the request).

    This is fixed by waiting for the "update" event before issuing the
    go-to-nearest request.

    </details>

### 🔨 Improvements

* <details><summary>Avoid changing the current date-time when selecting a product unless needed</summary>

    When the automatic jump-to-nearest request takes more time to load than the
    datasets, the user will see the current date-time change after the data has
    been load (partially or fully).
    That change is due to the automatic jump-to-nearest, which is supposed to
    unsure that the user has some data to see.

    This change avoid the unnecessary change by first checking if the visible
    data is sufficient.

    </details>

* <details><summary>Switch to two-digit years in coverage timeline with the viewport is too small</summary>

    If the browser's viewport is too small to fit the full years (4-digits) in
    the coverage timeline, they switch to the short (2-digit) version.
    The "too small" breakpoint was set to a fixed value of `720px`.
    But this value is not suitable for when there are a lot of years to display
    (eg. 40 years).

    This replaces the fix breakpoint with a dynamically calculated one based on
    the number of years to display.

    </details>

---

## 28-04-2021

### 🐛 Bug fixes

* Fix bug when trying to paste from clipboard on Firefox (and maybe others)

### 🔨 Improvements

* <details><summary>Replace short line segments in the detailed timeline with dots.</summary>

    In the detailed timeline (accessible by clicking the arrow right above the
    timeline)
    the dots were rendered when the dataset they represent had a time-span less then
    10 minutes.

    So a dataset with a timespan of 12 minutes used to be always rendered as a line
    segment even if it resulted in a line a few pixels long, which can be hard to
    click (eg. when the selected timespan is "Weekly").

    And a dataset with a timespan of 9 minutes used to be always rendered as a dot
    even if it would've resulted in a line segment of more then 100 pixels, which
    would more accurately show the start and end time of the dataset (eg. when the
    selected timespan is "1-hour").

    Now dots are used only if the line segment would've had a length shorter than
    5 pixels (the half of the dot's width).

    </details>

* Include the loading progress of external layers (marked with <svg width="10" height="10" fill="currentColor" stroke="currentColor"><path fill="none" d="M4 1.5h-2.5v8h8v-2.5m-5 -0.5l5 -5" /><path d="M6.5 1.5h3v3z" /></svg>)
in the loading bar.

---

## 04-03-2021

### 🎉 New features

* Replace products list with a catalog:
    * with a search field that filters on label and tags
    * and different grouping options based on tags
* Add support for showing the bearing when using the measuring ruler
* Add support for moving in time with left and right arrow keys
* Add "What's new!" dialog configured per portal
* Add support for highlighting products with selected datasets in the "Display data"
* Add <kbd>Shift</kbd>+<kbd>Z</kbd> to center/zoom on selected datasets
* Add support for duplicating shapes
* Add support for importing duplicate shapes
* Add support for showing basic info about selected dataset
* Add support for flipping a marker's image or polygon
* Add support for highlighting the processing tools button when it's active
* Add support for caching dataset remote API requests when using patterns
* Add support for painting the in-map datasets above the others in the extended timeline
* Add support for showing the coordinates of the map view's center when the mouse is not over the map

### 🐛 Bug fixes

* Fix bug when parsing user-provided shape date on iOS Safari
* Fix edge case bug in the WebGL streamlines renderer
* Fix long press on item in products list on iOS using Firefox Focus
* Fix bug where a product's datasets may still be visible even when they are out-of-zoom
* Fix edge cases when trying to find the nearest data

### 🔨 Improvements

* Replace products' all-directions drag handler with up-down drag handler
* Replace processing tools button icon with a better-suited one
* Prevent the coordinates view from being selectable on mobile
* Replace <kbd>Shift</kbd>+<kbd>Up</kbd>/<kbd>Down</kbd> with <kbd>PageUp</kbd>/<kbd>PageDown</kbd> to reorder products in "Display data"
* Improve the shortcuts descriptions
* Change <kbd>Ctrl</kbd>+<kbd>A</kbd> to select all visible *annotation shapes* instead of all visible *datasets*
* Improve duplicate shapes detection while importing
* Fix Google Maps patch to handle v3.42
* Avoid a useless forced repaint in the progress bar
* Avoid progress bar flicker
* Opacity gauge:
    * Update it to show the value and have more natural mouse/touch interactions
    * Add support for controlling it with a slider
    * Remove support for toggling it and for the <kbd>Left</kbd>, <kbd>Right</kbd> and <kbd>Backspace</kbd> keys

### ☠️ Feature removals

* Remove shortcuts that change the opacity of the selected datasets
* Remove <kbd>Shift</kbd>+<kbd>Up</kbd> shortcut to go to nearest data
