// Use global extent for web mercator
var maxExtent = [
  -20037508.342789, // left   (smallest longitude)
  -20037508.342789, // bottom (smallest latitude)
   20037508.342789, // right  (biggest longitude)
   20037508.342789, // top    (biggest latitude)
];
var tileOrigin = [maxExtent[0], maxExtent[3]];
var tileSize = [256, 256];

function createODLShortener(url) {
  return {
    method: 'POST',
    url: 'https://odl.bzh/a',
    data: {lsturl: url, format: 'json'},
    responseType: 'json',
    callback: function(response) {
      if (response.success) {
        return response.short;
      } else {
        return new Error(response.msg);
      }
    },
  };
}

module.exports = {
  version: '1.3.0',

  servers: [
    {
      id: 'my_server',
      serviceURL: 'https://MY_SERVER/webservice',
      dataURLs: [
        'https://MY_SERVER/tiles/',
      ],
      productIdMap: {
        /* MAP YOUR OWN PRODUCTS IDs HERE */
      },
    }
  ],
  shortenerForURL: createODLShortener,

  /* Map */
  projection: 'EPSG:3857',
  maxExtent: maxExtent,
  baseLayers: [
    new OpenLayers.Layer.WMS("GEBCO bathymetry",
      "https://www.gebco.net/data_and_products/gebco_web_services/web_map_service/mapserv",
      {
        "layers": "GEBCO_Grid",
        "version": "1.3.0",
        "format": "image/png",
      },
      {
        "isBaseLayer": true,
        "sphericalMercator": true,
        "numZoomLevels": 22,
        "wrapDateLine": true
      }
    ),
  ],
  additionalLayers: [
  ],

  /* Timeline */
  timeRanges: [
    ['6-Hour', '6h'],
    ['Daily', '1d', true],
    ['3-Day', '3d'],
    ['Weekly', '1w'],
    ['Bi-weekly', '2w'],
  ],

  /* Misc */
  storageId: 'MyPortal',
  markerSymbols: {
  },

  /* Predefined Data */
  messages: [
    {
      id: 'Sample',
      type: 'info',
      text: 'This is just a notification',
      showOnce: true,
    },
  ],
  hotspots: [
  ],
  drawableShapes: [
  ],
  products: [
    {
      label: '<i>User Shapes</i>',
      id: '900913_User_Shapes',
      type: 'USER_SHAPES',
      selected: true,
      mustBeCurrent: false,
      opacity: 1,
      stackLevel: 50,
      style: {
        ALL: {
          strokeColor: '#FFFFFF',
          fillOpacity: 0,
          fontSize: 16,
        },
        WKT: {
          strokeColor: '#AAAAEE',
          fillOpacity: 0.4,
          fontSize: 12,
        },
      },
    },

    /* ADD YOUR OWN PRODUCTS HERE */
  ],
};
