/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
/// <reference path='../_types.d.ts' />
import {Bounds, Geometry} from '../gis.js';
import {API} from './api.js';

declare module './api.js' {
  namespace API {
    interface DatasetInfo {
      productId: ProductId;
      datasetId: DatasetId;
      name: string;
      serverId: APIServerId;
      start: Date;
      end: Date;
      mapMaxZoom: number;
      uri: string;
      polygon: string;
      resolutions: unknown[];
      mapBounds: NSEW<string> | null | undefined;
      metadata?: JSONValue;
      bounds?: Bounds | null;
      geometry?: Geometry | null;
    }
    interface Nearest {
      past?: Date | null;
      future?: Date | null;
      nearest: Date | null;
    }
    interface ClearCacheOptions {
      datasets?: boolean;
      coverage?: boolean;
    }
  }
}

declare module './api_server.js' {
  namespace APIServer {
    interface DatasetDates {
      start: Date;
      end: Date;
    }
    interface Request {
      productIds: ProductId[];
      minDate?: Date;
      maxDate?: Date;
      extent?: LBRT | null;
      patternsByProductId: Record<ProductId, HybridPattern>;
      filtersByProductId: Record<ProductId, Record<string, boolean>>;
    }
    interface NearestRequest extends Request {
      date: Date;
      past: boolean;
      present: boolean;
      future: boolean;
    }
    interface NearestResult {
      past?: Array<Milliseconds | null>;
      future?: Array<Milliseconds | null>;
    }
    type DatasetInfo = Omit<API.DatasetInfo, 'serverId'>;
  }
}
