/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
/** */
let id = 0;

export class Feature {
  // #region Private typedefs

  /** @typedef {import('./gis_geometry.js').Geometry} Geometry */

  // #endregion
  // #region Public

  /**
   * @param {Geometry | null} geometry
   * @param {object | null} [properties]
   * @param {object | null} [style]
   */
  constructor(geometry, properties, style) {
    this.id = `feature_${id++}`;
    this.geometry = geometry || null;
    /** @type {object | null} */
    this.properties = Object.assign({}, properties);
    this.style = style || null;
    /** @type {any} */
    this.layer = null;
    /** @type {any} */
    this.dataset = null;

    this.renderIntent = 'default';
    /** @type {unknown} */
    this.layerInfo = null;
    /** @type {unknown} */
    this.rendererInfo = null;
  }

  destroy() {
    this.layer?.removeFeature(this);
    this.layer = null;
    this.properties = null;
    this.geometry = null;
    this.layerInfo = null;
    this.rendererInfo = null;
  }

  // #endregion
}
