/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {OpenLayers} from '../../vendors/openlayers.js';
import {BezierPath} from '../utils.js';
import {_GeometryFunctions, _ProjectionExtension} from './gis_geometry.js';
import {_GeometryExtension} from './gis_geometry_intersection.js';
import {_ProjectionFunctions} from './gis_projection.js';
import {OpenLayersMapExtension} from './gis_ol_patches.js';

export interface OLLonLat extends OpenLayers.LonLat {}
export interface OLBounds extends OpenLayers.Bounds {}
export interface OLSize extends OpenLayers.Size {}

export interface Point extends OpenLayers.Geometry.Point {
  __virtual?: boolean;
}
export interface LineString extends OpenLayers.Geometry.LineString {
  components: Point[];
}
export interface LinearRing extends OpenLayers.Geometry.LinearRing {
  components: Point[];
  __open?: boolean;
}
export interface Polygon extends OpenLayers.Geometry.Polygon {
  components: OneOrMore<LinearRing>;
  __bezier?: Readonly<BezierPath>;
  style?: object;
}
export interface MultiPoint extends OpenLayers.Geometry.MultiPoint {
  components: Point[];
}
export interface MultiLineString extends OpenLayers.Geometry.MultiLineString {
  components: LineString[];
}
export interface MultiPolygon extends OpenLayers.Geometry.MultiPolygon {
  components: Polygon[];
}

export type Polyline = LineString | LinearRing;

export interface GeometryCollection<T extends Geometry = Geometry> extends OpenLayers.Geometry.Collection<T> {}
export type Geometry = Point | Polyline | Polygon | GeometryCollection<Geometry>;

declare module './gis_geometry.js' {
  interface GeometryFunctions extends _GeometryFunctions {}
  interface GeometryFunctions extends _GeometryExtension {}
}
declare module './gis_projection.js' {
  interface ProjectionFunctions extends _ProjectionFunctions {}
  interface ProjectionFunctions extends _ProjectionExtension {}
}
declare module '../../vendors/openlayers.js' {
  namespace OpenLayers {
    interface Map extends OpenLayersMapExtension {}
    namespace Layer {
      interface Image {
        readonly tiles: Iterable<OpenLayers.Tile>;
      }
      interface Grid {
        readonly tiles: Iterable<OpenLayers.Tile>;
      }
    }
  }
}
