/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
/// <reference path='./_config.d.ts' />
/// <reference path='./_types.d.ts' />
/// <reference path='./syntool_.d.ts' />
/// <reference path='./map_view_.d.ts' />
import './polyfills.js';
import {
  configTypes,
  configFieldValidators,
  configValidators,
  configNormalizers,
  productFieldValidators,
  productNormalizers,
  assertValidConfig,
  normalizeConfig,
} from './config.js';
import * as RegExpOtherHelpers from './regexp.js';
import {RegExpHelpers, createInstanceProxy, raise} from './utils.js';
import {Syntool} from './syntool.js';
import './syntool_filters.js';
import './syntool_shortcuts.js';
import './types.js';

Object.assign(RegExpHelpers, RegExpOtherHelpers);
const Config = {
  get types() {
    return configTypes;
  },
  get productFieldValidators() {
    return productFieldValidators;
  },
  get productNormalizers() {
    return productNormalizers;
  },
  get configValidators() {
    return configValidators;
  },
  get configFieldValidators() {
    return configFieldValidators;
  },
  get configNormalizers() {
    return configNormalizers;
  },

  assertValid: assertValidConfig,
  normalize: normalizeConfig,
};

const SyntoolWithInstance = (
  /** @type {typeof Syntool & {instance: Syntool}} */ (Syntool)
);
SyntoolWithInstance.instance = new Syntool(
  document.getElementById('syntool-instance')
  ?? raise(new TypeError('Missing Syntool DOM node with id "syntool-instance"'))
);
const ProxySyntool = createInstanceProxy(SyntoolWithInstance);

export * from './api.js';
export * from './syntool_control.js';
export * from './constants.js';
export * from './coverage.js';
export * from './dataset.js';
export * from './gis.js';
export * from './quantity.js';
export * from './map_hub.js';
export * from './map_view.js';
export * from './notification_center.js';
export * from './pattern.js';
export * from './products_store.js';
export * from './shortcuts_store.js';
export {ProxySyntool as Syntool};
export * from './timeline_store.js';
export * from './user_storage.js';
export * from './utils.js';
export {Config};
