/* eslint-disable max-lines */
/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {OpenLayers} from '../vendors/openlayers.js';
import {ClassExtension} from './utils.js';
import {MapView} from './map_view.js';
import {MapViewDatasetsExtension} from './map_view_datasets.js';
import {Unit} from './quantity.js';

declare module './map_view.js' {
  namespace MapView {
    type ExtensionInterface<E> = Omit<E, keyof ClassExtension<any>>;

    type MapUnit = 'MapView.MapUnit';
    type MapUnits = InUnit<number, Unit> & InUnit<number, MapUnit>;
    interface Layer extends OpenLayers.Layer {}
    interface AdditionalLayer extends Layer {
      /**
       * When set to `true` the layer is displayed above all products. Otherwise
       * it's displayed under all products.
       *
       * Defaults: `false`
      */
      syntoolIsOverlay?: boolean;
    }
    interface Props {
      /**
       * A string identifying the Well Known Identifier for the projection.
       * Specifies the projection of the map.
       */
      projection: string;
      /**
       * The maximum extent of the map in the projection `projection`.
       */
      maxBounds: Projected<LBRT>;
      /**
       * The maximum navigable extent of the map in the projection `projection`.
       * If set to `null`, map navigation will be unrestricted.
       */
      restrictedBounds: Projected<LBRT>;
      /**
       * Some map providers (like Bing maps) skip some zoom levels (e.g. Bing
       * maps considers the zoom level `0` as level `1`), if it's the case you
       * can use this field to set the offset (e.g. set it to `1`).
       *
       * Default: `0`
       */
      zoomOffset: number;
      /**
       * A list of layers. It **MUST** contain at least one layer. One of these
       * layers (by default the first one) will be used as the background layer
       * of the map. These layers will be displayed in a list which and be
       * toggled using the globe button in the top bar of Syntool allowing the
       * user to switch between them.
       */
      baseLayers: OneOrMore<Layer>;
      /**
       * A list of layers that are always visible and placed above or below all
       * products.
       *
       * Default: `[]`
       */
      additionalLayers: AdditionalLayer[];
    }
    interface State {
      backgroundLayerName: string;
      extent: Projected<LBRT>;
      center: Projected<XY>;
      zoom: number;
    }
  }

  // @ts-expect-error
  interface MapView extends
    MapViewDatasetsExtension.InstanceFields,
    MapView.ExtensionInterface<MapViewDatasetsExtension> {}
  namespace MapView {
    interface Layer extends MapViewDatasetsExtension.Layer {}
    interface Props extends MapViewDatasetsExtension.Props {}
    interface State extends MapViewDatasetsExtension.State {}
  }
}

declare module '../vendors/openlayers.js' {
  namespace OpenLayers {
    interface Map {
      mapView?: MapView;
    }
  }
}
