/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Syntool} from "./syntool.js";
import {SyntoolControlsExtension} from "./syntool_control.js";
import {SyntoolFiltersExtension} from "./syntool_filters.js";
import {SyntoolShortcutsExtension} from "./syntool_shortcuts.js";
import {ProductsStore} from "./products_store.js";
import {MapView} from "./map_view.js";
import {TimelineStore} from "./timeline_store.js";

type _FiltersMap<M> = keyof M | {[x: keyof any]: _FiltersMap<M>};
type _Filters<M, T extends _FiltersMap<M>> = (
  T extends keyof M
  ? OmitThisParameter<M[T]> & (
    T extends string
    ? (
      `${T}WithId` extends keyof M
      ? {withId: OmitThisParameter<M[`${T}WithId`]>}
      : unknown
    )
    : unknown
  )
  // @ts-expect-error
  : {[K in keyof T]: _Filters<M, T[K]>}
);

type FILTERS = typeof Syntool.FILTERS;
type FiltersMap<E> = _FiltersMap<Omit<E, keyof Syntool.Extension>>;
type ExtensionFiltersMap<E extends Syntool.Extension> = Extract<E[FILTERS], FiltersMap<E>>;

declare module './syntool.js' {
  interface Syntool {
    filters: Syntool.Filters;
  }
  namespace Syntool {
    type ExtensionInterface<E extends Syntool.Extension> = Omit<E, keyof Syntool.Extension>;
    type ExtensionFiltersInterface<E extends Syntool.Extension> = _Filters<Omit<E, keyof Syntool.Extension>, ExtensionFiltersMap<E>>;

    interface Filters {}
    interface State extends ProductsStore.State {
      mapView: MapView.State;
      timeline: TimelineStore.State;
      geoFilterActive: boolean;
      colocActive: boolean;
    }
    interface PartialState extends ProductsStore.PartialState {
      mapView?: MapView.State;
      timeline?: TimelineStore.State;
      geoFilterActive?: boolean;
      colocActive?: boolean;
    }
  }

  interface Syntool extends
    SyntoolControlsExtension.InstanceFields,
    Syntool.ExtensionInterface<SyntoolControlsExtension> {}
  namespace Syntool {
    interface State extends SyntoolControlsExtension.State {}
    interface PartialState extends Partial<SyntoolControlsExtension.State> {}
  }

  interface Syntool extends Syntool.ExtensionInterface<SyntoolFiltersExtension> {}
  namespace Syntool {
    interface Filters extends Syntool.ExtensionFiltersInterface<SyntoolFiltersExtension> {}
  }

  interface Syntool extends
    SyntoolShortcutsExtension.InstanceFields,
    Syntool.ExtensionInterface<SyntoolShortcutsExtension> {}
  namespace Syntool {
    interface State extends SyntoolShortcutsExtension.State {}
    interface PartialState extends Partial<SyntoolShortcutsExtension.State> {}
  }
}
