/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {test, assert} from './test_utils.js';
import {ajax} from './ajax.js';
import {isAbortError} from './utils.js';

test('Ajax - pre-aborted', function() {
  var a = new AbortController();
  a.abort();
  var timeoutID = setTimeout(assert.bind(null, false, 'timed-out'), 4000);
  ajax('https://httpbin.org/status/200', {signal: a.signal})
  .then(function() {
    assert(false);
  })
  .catch(function(error) {
    assert(isAbortError(error));
  })
  .finally(function() {
    clearTimeout(timeoutID);
  });
});
test('Ajax - in-aborted', function() {
  var a = new AbortController();
  setTimeout(a.abort.bind(a), 2000);
  var timeoutID = setTimeout(assert.bind(null, false, 'timed-out'), 4000);
  ajax('https://httpbin.org/delay/5', {signal: a.signal})
  .then(function() {
    assert(false);
  })
  .catch(function(error) {
    assert(isAbortError(error));
  })
  .finally(function() {
    clearTimeout(timeoutID);
  });
});
test('Ajax - post-aborted', function() {
  var a = new AbortController();
  setTimeout(a.abort.bind(a), 3000);
  var timeoutID = setTimeout(assert.bind(null, false, 'timed-out'), 4000);
  ajax('https://httpbin.org/delay/1', {signal: a.signal})
  .catch(function(error) {
    assert(!isAbortError(error));
  })
  .finally(function() {
    clearTimeout(timeoutID);
  });
});
test('Ajax - not-aborted', function() {
  var a = new AbortController();
  var timeoutID = setTimeout(assert.bind(null, false, 'timed-out'), 4000);
  ajax('https://httpbin.org/delay/1', {signal: a.signal})
  .catch(function(error) {
    assert(!isAbortError(error));
  })
  .finally(function() {
    clearTimeout(timeoutID);
  });
});
