/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/* From Stack Overflow
 * Original question: https://stackoverflow.com/questions/9847580/how-to-detect-safari-chrome-ie-firefox-and-opera-browser
 * Question author: user1289292 - https://stackoverflow.com/users/1289292/user1289292
 * Answer: https://stackoverflow.com/a/9851769
 * Answer author: Rob W - https://stackoverflow.com/users/938089
 *
 * With the modification to support detecting Safari 10
 * Original question: https://stackoverflow.com/q/39120772
 * Question author: catico - https://stackoverflow.com/users/5810034/catico
 * Answer: https://stackoverflow.com/a/39621764
 * Answer author: virtualcodewarrior - https://stackoverflow.com/users/6356268/virtualcodewarrior
 */

// Opera 8.0+
export const isOpera = (
  // @ts-expect-error
  (!!window.opr && !!window.opr.addons) ||
  // @ts-expect-error
  !!window.opera || navigator.userAgent.indexOf(' OPR/') >= 0
);

// Firefox 1.0+
/** @type {boolean} */
export const isFirefox = (
  // @ts-expect-error
  (window.CSS2Properties && 'MozAnimation' in window.CSS2Properties.prototype) ||
  // @ts-expect-error
  typeof InstallTrigger !== 'undefined'
);

// Internet Explorer 6-11
// @ts-expect-error
export const isIE = /*@cc_on!@*/false || !!document.documentMode;

// Edge 20+
// @ts-expect-error
export const isEdge = !isIE && !!window.StyleMedia;

// Chrome 1+
// @ts-expect-error
export const isChrome = !!(window.chrome && (window.chrome.webstore || window.chrome.csi));

// Blink engine detection
export const isBlink = (isChrome || isOpera) && !!window.CSS;

/* From Stack Overflow
 * Original question: https://stackoverflow.com/q/10527983
 * Question author: alt - https://stackoverflow.com/users/743561/alt
 * Answer: https://stackoverflow.com/a/11752084
 * Answer author: Vitim.us - https://stackoverflow.com/users/938822/vitim-us
 *
 * For a list of possible values see http://stackoverflow.com/a/19883965/2168919
 */

// Windows
export const isWindows = navigator.platform.toLowerCase().indexOf('win') === 0;

// macOS
export const isMacOS = navigator.platform.toLowerCase().indexOf('mac') === 0;

// iOS
export const isIOS = /(iPhone|iPod|iPad)/i.test(navigator.platform);

export const isApple = isMacOS || isIOS;

// Safari
export const isSafari = (
  // Safari <= 9 "[object HTMLElementConstructor]"
  Object.prototype.toString.call(window.HTMLElement).indexOf('Constructor') > 0 ||
  // Safari 10
  // @ts-expect-error
  (!isChrome && !isOpera && window.webkitAudioContext !== undefined) ||
  // Only Safari on iOS
  isIOS ||
  // fallback
  (isMacOS && !isChrome && !isOpera && navigator.userAgent.toLowerCase().indexOf('safari/') >= 0)
);

export const isLittleEndian = (function() {
  /* https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Global_Objects/DataView#Endianness */
  if (window.ArrayBuffer) {
    var buffer = new ArrayBuffer(2);

    new DataView(buffer).setInt16(0, 256, true /* littleEndian */);

    // Int16Array uses the platform's endianness.
    return new Int16Array(buffer)[0] === 256;
  }
  return false;
})();
// export const isLittleEndian = (function() {
//   var u16array = new Uint16Array([0x1234]);
//   var u8array = new Uint8Array(u16array.buffer);
//   return u8array[0] === 0x34;
// }());
export const isBigEndian = !isLittleEndian;
