/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
/** */
export const whiteSpace = '\\s';
export const requiredWhiteSpace = whiteSpace + '+';
export const optionalWhiteSpace = whiteSpace + '*';

export const sign = '[+-]?';
export const hexDigit = '[0-9a-fA-F]';
export const uint = '\\d+';
export const int = sign + uint;
export const uint8 = '(?:0*(?:2(?:[01234]\\d|5[0-5])|1?\\d{0,2}|0))';
export const ufloat = '(?:\\.\\d+|\\d+(?:\\.\\d*)?)';
export const ieeeUfloat = '(?:' + ufloat + '(?:[eE]' + int + ')?)';
export const float = sign + ufloat;
export const ieeeFloat = sign + ieeeUfloat;

export const cssPixels = ieeeFloat + 'px';

var dot = '\\.';
export const ipAddress = uint8 + dot + uint8 + dot + uint8 + dot + uint8;
var urlDomainAlphanum = '[a-z\\u00a1-\\uffff0-9]';
var urlDomainAlpha = '[a-z\\u00a1-\\uffff]';
var urlDomainPart = list(urlDomainAlphanum + '+', '-');
var urlDomainTld = urlDomainAlpha + '{2,}';
var urlScheme = '(?:https?:)?';
var urlUser = '(?:\\S+(?::\\S*)?@)?';
var urlDomainName = list(urlDomainPart, dot) + dot + urlDomainTld;
var urlHost = or(ipAddress, urlDomainName);
var urlPort = '(?::\\d{2,5})?';
var urlPath = '(?:/?[^\\s?#]*)?';
var urlQuery = '(?:\\?[^\\s#]*)?';
var urlFragment = '(?:#[^\\s]*)?';
export const url = [
  urlScheme,
  '(?://', urlUser, urlHost, urlPort, ')?',
  urlPath, urlQuery, urlFragment,
].join('');

var wktCouple = '(?:' + ieeeFloat + requiredWhiteSpace + ieeeFloat + ')';
var wktPointArg = funcArgs(wktCouple);
var wktLinestringArg = funcVaArgs(wktCouple);
var wktPolygonArg = funcVaArgs(wktLinestringArg);
var wktMultipointArg = or(funcVaArgs(wktPointArg), funcVaArgs(wktCouple));
var wktMultilinestringArg = funcVaArgs(wktLinestringArg);
var wktMultipolygonArg = funcVaArgs(wktPolygonArg);

/*eslint-disable comma-spacing, array-bracket-spacing */
export const wktPoint           = ['POINT'          , wktPointArg          ].join(optionalWhiteSpace);
export const wktLinestring      = ['LINESTRING'     , wktLinestringArg     ].join(optionalWhiteSpace);
export const wktPolygon         = ['POLYGON'        , wktPolygonArg        ].join(optionalWhiteSpace);
export const wktMultipoint      = ['MULTIPOINT'     , wktMultipointArg     ].join(optionalWhiteSpace);
export const wktMultilinestring = ['MULTILINESTRING', wktMultilinestringArg].join(optionalWhiteSpace);
export const wktMultipolygon    = ['MULTIPOLYGON'   , wktMultipolygonArg   ].join(optionalWhiteSpace);
/*eslint-enable comma-spacing, array-bracket-spacing */
export const wktGeometry = or(
  wktPoint,
  wktLinestring,
  wktPolygon,
  wktMultipoint,
  wktMultilinestring,
  wktMultipolygon
);

/**
 * @param {string[]} args
 * @returns {string}
 */
export function or(...args) {
  return '(?:' + args.join('|') + ')';
}
/**
 * @param {string} element
 * @param {string} separator
 * @returns {string}
 */
export function list(element, separator) {
  return '(?:' + element + '(?:' + separator + element + ')*)';
}
/**
 * @param {string[]} args
 * @returns {string}
 */
export function funcArgs(...args) {
  return (
    '\\(' +
    optionalWhiteSpace +
    args.join(optionalWhiteSpace + ',' + optionalWhiteSpace) +
    optionalWhiteSpace +
    '\\)'
  );
}
/**
 * @param {string} arg
 * @returns {string}
 */
export function funcVaArgs(arg) {
  return (
    '\\(' + list(optionalWhiteSpace + arg + optionalWhiteSpace, ',') + '\\)'
  );
}
/**
 * @param {string} name
 * @param {string[]} args
 * @returns {string}
 */
export function funcCall(name, ...args) {
  return name + optionalWhiteSpace + funcArgs(...args);
}

/**
 * @param {string} string
 * @returns {string}
 */
export function escape(string) {
  return string.replace(/[\\^$.*+?()[\]{}|]/g, '\\$&');
}
