/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {} from './scheduling.js';

declare module './scheduling.js' {
  interface DebouncedFunction<F extends (...args: any[]) => any> {
    (this: ThisParameterType<F>, ...args: Parameters<F>): void;
    /**
     * Indicated whether or not an invocation of the debounced function has been
     * scheduled.
     */
    readonly scheduled: boolean;
    /**
     * Cancels the invocation of the debounced function.
     */
    cancel(): void;
    /**
     * Invokes the debounced function immediately.
     */
    invoke(): void;
  }
  interface ThrottledFunction<F extends (...args: any[]) => any> {
    (this: ThisParameterType<F>, ...args: Parameters<F>): void;
    /**
     * Indicated whether or not an invocation of the throttled function has been
     * scheduled.
     */
    readonly scheduled: boolean;
    /**
     * Cancels the invocation of the throttled function.
     */
    cancel(): void;
    /**
     * Invokes the throttled function immediately.
     */
    invoke(): void;
  }
}
