/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Dataset} from '../../syntool.js';
import {withColorPalette} from '../../utils/color_palette.js';
import '../../features/product_colormaps.js';

export const colorPaletteExtension = {
  meta: {name: 'ColorPaletteFromColormapsExtension'},
  initialize(info) {
    window.original();
    const product = this.getProduct();
    this.colorPaletteURL = product.colormaps?.[0]?.colorPalette;
    this.colorPalette = null;
  },
  hasResources() {
    return window.original() || !!this.colorPalette;
  },
  destroy() {
    this.colorPaletteRequested = false;
    this.colorPalette = null;
    window.original();
  },

  withColorPalette(callback) {
    if (!this.colorPaletteURL || this.colorPalette) {
      callback?.(this.colorPalette);
      return this.colorPalette;
    }

    return withColorPalette(this.colorPaletteURL, (colorPalette) => {
      this.colorPalette = colorPalette;
      callback?.(colorPalette);
    });
  },
};

Object.assign(Dataset.extensions, {colorPalette: colorPaletteExtension});
