/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {throttleOncePerEventLoop, Dataset} from '../../syntool.js';
import {layerExtension} from './layer_extension.js';

const layerManagersByProduct = new WeakMap();
export const sharedLayerExtension = {
  meta: {
    name: 'SharedLayerExtension',
    deps: [layerExtension],
  },
  static: {
    layerManagersByProduct,
  },
  createLayerManager() {
    const product = this.getProduct();
    let layerManager = layerManagersByProduct.get(product);
    if (!layerManager) {
      layerManager = window.original();
      layerManagersByProduct.set(product, layerManager);

      const handleProductSelectionChange = throttleOncePerEventLoop(() => {
        layerManager.setHidden?.(
          !this.syntool.productsStore.isProductSelected(product.id)
        );
      });

      this.syntool.productsStore.hub
        .on('change:selection', handleProductSelectionChange);
    }
    return layerManager;
  },
};

Object.assign(Dataset.extensions, {sharedLayer: sharedLayerExtension});
