/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {wrapFn, Dataset} from '../../syntool.js';
import {OLVectorLayer} from '../../utils/ol_vector_layer.js';
import {VectorLayerManager} from '../layer_managers/vector_layer_manager.js';
import {layerExtension} from './layer_extension.js';

export const vectorLayerExtension = {
  meta: {
    name: 'VectorLayerExtension',
    deps: [layerExtension],
  },
  static() {
    this.normalizeProductConfig = (
      this.normalizeProductConfig
      ? wrapFn(this.normalizeProductConfig, function(productConfig) {
        return window.original(normalizeProductConfig(productConfig));
      })
      : normalizeProductConfig
    );

    function normalizeProductConfig(productConfig) {
      if (
        !('outlines' in productConfig) &&
        !('noOutline' in productConfig)
      ) {
        productConfig.outlines = 'HIDDEN_IN_ZOOM_RANGE';
      }
      return productConfig;
    }
  },

  createLayer() {
    return new OLVectorLayer(
      this.name,
      Object.assign(
        {mapMoveRedrawMode: OLVectorLayer.MapMoveRedrawMode.RENDERED},
        this.getVectorLayerCreationOptions?.()
      )
    );
  },
  createLayerManager() {
    return (
      new VectorLayerManager(() => this.createLayer(), this.syntool.mapView)
    );
  },
  show() {
    window.original();
    if (this.syntool.mapView.isDatasetInMap(this)) {
      this.layerManager.redraw(this);
    }
  },
};

Object.assign(Dataset.extensions, {vectorLayer: vectorLayerExtension});
