/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Dataset, Types} from '../syntool.js';
import {layerExtension} from './extensions/layer_extension.js';
import {vectorLayerExtension} from './extensions/vector_layer_extension.js';
import {metadataExtension} from './extensions/metadata_extension.js';
import {dataFieldsExtension} from './extensions/data_fields_extension.js';
import {auxiliaryDataExtension} from './extensions/auxiliary_data_extension.js';
import {dataLoadingExtension} from './extensions/data_loading_extension.js';
import {colorPaletteExtension} from './extensions/color_palette_extension.js';
import {geoJsonExtension} from './extensions/geojson_extension.js';
import {sharedLayerExtension} from './extensions/shared_layer_extension.js';

const geoJsonDatasetExtension = {
  getVectorLayerCreationOptions() {
    return {
      style: {
        base: {
          stroke        : true,
          strokeOpacity : 1,
          strokeWidth   : 2,
          fill          : true,
          fillOpacity   : 1,
          graphicOpacity: 1,
          pointRadius   : 4,
        },
      },
    };
  },
  isGlobal() {
    return false;
  },
  handleFeaturesLoad(features) {
    const {promise, resolve} = Promise.withResolvers();

    this.updateFeaturesStyle(features, () => {
      this.layerManager.addFeatures(this, features);
      setTimeout(() => resolve(), 100);
    });
    return promise;
  },
};

export const GeoJSONDataset = Dataset.createClass({
  static: {
    name        : 'GeoJSONDataset',
    productTypes: ['GEOJSON'],
    renderFields: {
      color: ['ref', Types.Color],
    },
  },
  extensions: [
    layerExtension,
    vectorLayerExtension,
    metadataExtension,
    dataFieldsExtension,
    auxiliaryDataExtension,
    dataLoadingExtension,
    colorPaletteExtension,
    sharedLayerExtension,
    geoJsonExtension,
    geoJsonDatasetExtension,
  ],
});
window.ODL = Object.assign(window.ODL || {}, {GeoJSONDataset});
