/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {LocalAPIServer} from '../../syntool.js';

const productInfosCache = new WeakMap();

export class LayerAPIServer extends LocalAPIServer {
  /*Public*/
  handlesAllProductsOfType(type) {
    return type === 'LAYER';
  }

  /*Protected*/
  _getAllInfosForProductWithId(productId) {
    const product = this.api.syntool.productsStore.getProduct(productId);
    if (!productInfosCache.has(product)) {
      productInfosCache.set(product, [
        {
          productId: productId,
          datasetId: productId + '-layer_dataset',
          name     : product.layer.name,
          start    : product.validFrom,
          end      : product.validTill,
        },
      ]);
    }
    return productInfosCache.get(product);
  }
}
