/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import '../map_view_extensions/map_view_clickable_features.js';
import {
  deepAssign,
  Bounds,
  Geometry,
  Feature,
  Config,
  Dataset,
  Types,
} from '../syntool.js';
import {metadataExtension} from './extensions/metadata_extension.js';
import {auxiliaryDataExtension} from './extensions/auxiliary_data_extension.js';

var mooredExtension = {
  initialize(info) {
    window.original();
    this._stackLevel = 0;
    this.iconFeature = null;
  },
  getStackLevel() {
    return this._stackLevel;
  },
  setStackLevel(stackLevel) {
    if (stackLevel !== this._stackLevel) {
      this._stackLevel = stackLevel;
      this.hub.emit('change:stackLevel');
    }
  },
  isGlobal() {
    return false;
  },
  didAddToMap() {
    window.original();
    if (!this.iconFeature) {
      this.iconFeature = this.createMarkerFeature();
      this.syntool.mapView.addClickableFeature(this, this.iconFeature);
    }
  },
  destroy() {
    this.iconFeature = null;
    window.original();
  },

  createMarkerFeature() {
    const product = this.getProduct();
    const markerStyle = deepAssign({}, product.marker);
    return new Feature(
      Geometry.createPointFromXY(Bounds.getCenter(this.bounds)),
      {markerStyle}
    );
  }
};

export const MooredDataset = Dataset.createClass({
  static: {
    name          : 'MooredDataset',
    productTypes  : ['MOORED'],
    validateConfig: Types.shape({
      marker: Config.types.MarkerStyle,
    }),
    normalizeProductConfig(productConfig) {
      productConfig.marker = (
        Config.applyMarkerStyleDefaults(productConfig.marker || {
          default: {
            externalGraphic: 'images/buoy-icon.png',
            graphicWidth   : 32,
            graphicHeight  : 32,
          },
        })
      );
      return productConfig;
    },
  },
  extensions: [
    metadataExtension,
    auxiliaryDataExtension,
    mooredExtension,
  ],
});
window.ODL = Object.assign(window.ODL || {}, {MooredDataset});
