/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Dataset, Types} from '../syntool.js';
import {getGridLayerOptions} from '../utils/ol_utils.js';
import {OpenLayers} from '../vendors/openlayers.js';
import {layerExtension} from './extensions/layer_extension.js';
import {metadataExtension} from './extensions/metadata_extension.js';
import {auxiliaryDataExtension} from './extensions/auxiliary_data_extension.js';
import {sharedLayerExtension} from './extensions/shared_layer_extension.js';

var ncwmsExtension = {
  createLayer() {
    var product = this.getProduct();
    var datasetSettings = this.getAdaptedSettings();
    var layerParams = Object.assign(
      {},
      product.ncwmsParams,
      datasetSettings.params
    );
    var layerOptions = Object.assign(getGridLayerOptions(), {
      //////////////
      // Options  //
      //////////////

      /* OpenLayers.Layer */
      maxExtent: this.bounds,
    });

    var url = product.ncwmsURL;
    if (datasetSettings.url) {
      url = datasetSettings.url;
    }

    return new OpenLayers.Layer.WMS(this.name, url, layerParams, layerOptions);
  },
  didAddToMap() {
    window.original();

    var datasetSettings = this.getAdaptedSettings();
    if (
      datasetSettings.url &&
      datasetSettings.url !== this.layerManager.layer.url
    ) {
      this.layerManager.layer.url = datasetSettings.url;
    }
    if (this.layerManager.layer) {
      this.layerManager.layer.mergeNewParams(datasetSettings.params);
    }
  },

  getAdaptedSettings() {
    var product = this.getProduct();
    var mid = new Date((this.start.getTime() + this.end.getTime()) / 2);
    var params = {TIME: mid.toISOString()};
    var url = null;
    if (product.ncwmsQueryAdapter) {
      var datasetSettings = product.ncwmsQueryAdapter(this);
      url = datasetSettings.url;
      params = Object.assign(params, datasetSettings.params);
    }

    return {url: url, params: params};
  },
};

export const NCWMSDataset = Dataset.createClass({
  static: {
    name          : 'NCWMSDataset',
    productTypes  : ['NCWMS'],
    validateConfig: Types.shape({
      ncwmsURL         : Types.URL.isRequired,
      ncwmsParams      : Types.shape({}),
      ncwmsQueryAdapter: Types.func,
    }),
  },
  extensions: [
    layerExtension,
    metadataExtension,
    auxiliaryDataExtension,
    sharedLayerExtension,
    ncwmsExtension,
  ],
});
window.ODL = Object.assign(window.ODL || {}, {NCWMSDataset});
