/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Syntool} from '../../syntool.js';

Syntool.extend({
  [Syntool.NAME]: 'ShapeFilters',
  [Syntool.FILTERS]: /** @type {const} */ ({
    product: {
      shapesProduct      : 'filterProductShapesProduct',
      serverShapesProduct: 'filterProductServerShapesProduct',
      userShapesProduct  : 'filterProductUserShapesProduct',
    },
    dataset: {
      isShape        : 'filterDatasetIsShape',
      isServerShape  : 'filterDatasetIsServerShape',
      isUserShape    : 'filterDatasetIsUserShape',
      isReadOnlyShape: 'filterDatasetIsReadOnlyShape',
      isEditableShape: 'filterDatasetIsEditableShape',
    },
  }),

  filterProductShapesProduct(product) {
    return (
      product.type === 'USER_SHAPES' ||
      product.type === 'SERVER_SHAPES'
    );
  },
  filterProductShapesProductWithId(productId) {
    return this.filterProductShapesProduct(
      this.productsStore.getProduct(productId)
    );
  },
  filterProductServerShapesProduct(product) {
    return product.type === 'SERVER_SHAPES';
  },
  filterProductServerShapesProductWithId(productId) {
    return this.filterProductServerShapesProduct(
      this.productsStore.getProduct(productId)
    );
  },
  filterProductUserShapesProduct(product) {
    return this.filterProductUserShapesProductWithId(product.id);
  },
  filterProductUserShapesProductWithId(productId) {
    return (
      this.userShapesProduct &&
      productId === this.userShapesProduct.id
    );
  },

  filterDatasetIsShape(dataset) {
    /*Use ODL.ShapeDataset to avoid circular dependencies*/
    return dataset instanceof ODL.ShapeDataset;
  },
  filterDatasetIsServerShape(dataset) {
    return this.filterProductServerShapesProductWithId(dataset.productId);
  },
  filterDatasetIsUserShape(dataset) {
    return this.filterProductUserShapesProductWithId(dataset.productId);
  },
  filterDatasetIsReadOnlyShape(dataset) {
    return this.filterDatasetIsShape(dataset) && dataset.readOnly;
  },
  filterDatasetIsEditableShape(dataset) {
    return this.filterDatasetIsShape(dataset) && !dataset.readOnly;
  },
});
