/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Dataset, Types} from '../syntool.js';
import {layerExtension} from './extensions/layer_extension.js';
import {sharedLayerExtension} from './extensions/shared_layer_extension.js';

var timeLayerExtension = {
  createLayer() {
    return this.getProduct().layer;
  },
  handleVisibilityChange() {
    window.original();
    if (this.isVisible()) {
      this.getProduct().updateLayer(
        this.layerManager.layer,
        this.syntool.timelineStore.currentDate
      );
    }
  },
};

export const TimeLayerDataset = Dataset.createClass({
  static: {
    name          : 'TimeLayerDataset',
    productTypes  : ['TIMELAYER'],
    validateConfig: Types.allOfType([
      Types.shape({
        layer      : Types.Layer.isRequired,
        updateLayer: Types.func.isRequired,
      }),
    ]),
  },
  extensions: [
    layerExtension,
    sharedLayerExtension,
    timeLayerExtension,
  ],
});
window.ODL = Object.assign(window.ODL || {}, {TimeLayerDataset});
