/**
 * @author: <sylvain.herledan@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {FlateStream} from '../../vendors/zlib.js';

export class TilesMask {
  constructor() {
    this._masks = null;
    this._offsets = null;
    this._minZoom = null;
    this._maxZoom = null;
  }

  init(minZoom, maxZoom, encodedData) {
    var decoded; //
    var bytes; //
    var decompressor;  //
    var i; //

    decoded = atob(encodedData);
    bytes = decoded.split('').map(function(x) {
      return x.charCodeAt(0);
    });
    decompressor = new FlateStream(bytes);
    this._masks = decompressor.getBytes();
    this._minZoom = minZoom;
    this._maxZoom = maxZoom;
    this._offsets = {};
    this._offsets[minZoom] = 0;
    for (i = minZoom + 1; i <= maxZoom; ++i) {
      this._offsets[i] = this._offsets[i - 1] + Math.pow(4, i - 1);
    }
  }

  hasTile(z, x, y) {
    var i; //
    var byteIndex; //
    var bit; //

    if (z < this._minZoom || z > this._maxZoom) {
      return false;
    }

    i = this._offsets[z] + y * Math.pow(2, z) + x;
    byteIndex = (i / 8) | 0;
    //eslint-disable-next-line no-bitwise
    bit = (this._masks[byteIndex] >>> (i % 8)) & 1;
    return 0 < bit;
  }
}
