/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Syntool} from '../syntool.js';
import {CatalogStore} from '../utils/catalog_store.js';

Syntool.extend({
  [Syntool.NAME]: 'Catalog',
  [Syntool.STATE]: {
    get() {
      const {selection} = this.catalogStore.state;
      const catalogSelection = selection.map((productId) => (
        this.productsStore.isProductSelected(productId) ? '1' : '0'
      )).join('');

      return {selection, catalogSelection};
    },
    set({selection = [], catalogSelection}) {
      this.catalogStore.selectOnlyProducts(selection);
      if (catalogSelection) {
        this.catalogStore.state = {
          selection: selection.filter((_, i) => catalogSelection[i] === '1'),
        };
      }
    },
  },

  get catalogStore() {
    Object.defineProperty(this, 'catalogStore', {
      value: new CatalogStore(this.productsStore),
      writable: true,
      enumerable: true,
      configurable: true,
    });
    this.catalogStore.hub.on('change:state', this._invalidateState, this);
    return this.catalogStore;
  },
  set catalogStore(store) {
    // When the getter is called the first time, it will replace the
    // getter/setter with an normal value property
    var prevStore = this.catalogStore;

    if (prevStore) {
      prevStore.hub.off('change:state', this._invalidateState, this);
    }

    // This doesn't call this setter because it has been replaced by the first
    // call to the getter
    this.catalogStore = store;

    if (store) {
      store.hub.on('change:state', this._invalidateState, this);
    }
  },
});

window.addEventListener('DOMContentLoaded', function() {
  if (ODL.appStateFields) {
    Object.assign(ODL.appStateFields, {
      selection: new ODL.appStateFieldTypes.StringField({
        get(state) {
          return state.catalogSelection;
        },
        set(state, catalogSelection) {
          return Object.assign(state, {catalogSelection});
        },
      }),
    });
  }
});
