/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Syntool} from '../syntool.js';
import {downloadAsFile} from '../utils/download_as_file.js';
import './syntool_clipboard.js';

const JSON_MIME_TYPE = 'application/json';

Syntool.extend({
  [Syntool.NAME]: 'Export',

  exportText(text, options) {
    options = Object.assign({
      label   : 'the export',
      fileName: 'syntool-export-' + (new Date().toISOString()) + '.txt',
      fileType: 'text/plain',
    }, options);

    if (options.fileName) {
      downloadAsFile(text, options.fileName, options.fileType);
    } else {
      this.copyToClipboard(text, options);
    }
  },
  exportJSON(json, options) {
    this.exportText(JSON.stringify(json, null, 2), Object.assign({
      label   : 'the export',
      fileName: 'syntool-export-' + (new Date().toISOString()) + '.json',
      fileType: JSON_MIME_TYPE,
    }, options));
  },
});
