/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {
  emptyFunction,
  forEachObject,
  whenReady,
  UserStorage,
  Syntool,
} from '../syntool.js';

var userStorage = new UserStorage('FlagsStore', 'flags');
userStorage.prefix = 'FlagsStore.';

var flags = new Map();

export const FlagsStore = {
  migrateCookies: migrateCookies,
  load          : loadFlags,
  has           : hasFlag,
  get           : getFlag,
  set           : setFlag,
  delete        : deleteFlag,
  toJSON        : toJSON,
  fromJSON      : fromJSON,
};

window.ODL = Object.assign(window.ODL || {}, {FlagsStore});

whenReady(function() {
  if (Syntool.backupStoresMap) {
    Syntool.backupStoresMap.flags = FlagsStore;
  }
});

function decode(string) {
  return decodeURIComponent(string.replace('+', ' '));
}
function migrateCookie(cookie) {
  var parts = cookie.split('=');
  var encodedName = parts.shift().trim();
  var name = decode(encodedName);
  var value = decode(parts.join('=').trim());

  return (
    _setFlag(name, value === 'on')
    .then(function deleteCookie() {
      document.cookie = encodedName + '=; expires=Thu, 01 Jan 1970 00:00:00 GMT';
    }, emptyFunction)
  );
}

function loadFlagsIterator(value, key) {
  flags.set(key, value);
}

function migrateCookies() {
  var cookies = document.cookie;

  if (!cookies) {
    return Promise.resolve();
  }
  return Promise.all(cookies.split('; ').map(migrateCookie));
}
function loadFlags() {
  flags.clear();
  return userStorage.forEach(loadFlagsIterator, 'load flags').catch(emptyFunction);
}
function hasFlag(key) {
  return flags.has(key);
}
function getFlag(key) {
  return flags.get(key);
}
function setFlag(key, value) {
  _setFlag(key, value).catch(emptyFunction);
}
function deleteFlag(key) {
  _deleteFlag(key).catch(emptyFunction);
}
function _setFlag(key, value) {
  flags.set(key, value);
  return userStorage.set(key, value);
}
function _deleteFlag(key) {
  flags.delete(key);
  return userStorage.delete(key);
}

function toJSON() {
  return Object.fromEntries(flags);
}
function fromJSON(json) {
  forEachObject(json, function(value, key) {
    setFlag(key, value);
  });
}
