/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {delay, ajaxHub, ajaxStatistics, Syntool} from '../syntool.js';

Syntool.extend({
  [Syntool.NAME]: 'MonitorLoadingTiles',
  [Syntool.INIT]() {
    const tilesLoading = new Set();
    function monitorLayer(layer) {
      layer.events.on({tileloadstart, tileloaded});
      for (const tile of layer.tiles || []) {
        if (tile?.isLoading) {
          addLoadingTile(tile);
        }
      }
    }
    function unmonitorLayer(layer) {
      for (const tile of tilesLoading) {
        if (tile.layer === layer) {
          removeLoadingTile(tile);
        }
      }
      layer.events.un({tileloadstart, tileloaded});
    }

    this.mapView.hub
      .on('layer:will-add-to-map', monitorLayer)
      .on('layer:did-remove-from-map', unmonitorLayer);
    this.mapView.getLayers().forEach(monitorLayer);

    function tileloadstart({tile}) {
      addLoadingTile(tile);
    }
    async function tileloaded({tile}) {
      try {
        await tile.getImage().decode();
        await delay(100);
      } catch (error) {
        /* ignore error */
      }
      removeLoadingTile(tile);
    }
    function addLoadingTile(tile) {
      if (!tilesLoading.has(tile)) {
        ajaxStatistics.liveRequestCount++;
        ajaxHub.emit('pre-send');
        tilesLoading.add(tile);
        return true;
      }
      return false;
    }
    function removeLoadingTile(tile) {
      if (tilesLoading.delete(tile)) {
        ajaxStatistics.liveRequestCount--;
        ajaxHub.emit('success');
        return true;
      }
      return false;
    }
  },
});
