/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Types as is, Config} from '../syntool.js';

// #region Exported typedefs

const ColormapTick = is.tuple([
  is.number.gte(0).lte(1).isRequired,
  is.oneOfType([is.number, is.string]).isRequired,
]);
const Colormap = is.shape({
  label       : is.string,
  colorPalette: is.URL.isRequired,
  ticks       : is.arrayOf(ColormapTick.isRequired).isRequired,
});

Object.assign(is, {
  ColormapTick,
  Colormap,
});
Object.assign(Config.productFieldValidators, {
  colormaps: is.arrayOf(Colormap.isRequired),
});

/**
 * @typedef {[position: number, value: number | string]} ColormapTick
 */
/**
 * @typedef Colormap
 * @prop {string} [label]
 * @prop {string | URL} colorPalette
 * @prop {ColormapTick[]} ticks
 */

// #endregion
