/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Shape} from '../../datasets/shape_dataset/shape.js';
import {Geometry, Feature} from '../../syntool.js';
import {PointHandler} from './point_handler.js';

const originPoint = Geometry.createPoint(0, 0);

export class OLGeometryHandler extends PointHandler {
  /*Public*/
  constructor(options) {
    super(options);
    this.type = Shape.WKT;
    this.geometry = options?.geometry ?? originPoint;
  }

  /*Protected*/
  getGeometry() {
    return this.point && this.point.geometry;
  }

  modifyFeature(projectedXY) {
    if (this.point) {
      const lastLonlat = this.point.geometry.getBounds().getCenterLonLat();
      this.point.geometry.move(
        projectedXY[0] - lastLonlat.lon,
        projectedXY[1] - lastLonlat.lat
      );
      this.point.geometry.clearBounds();
      this.layer.drawFeature(this.point, true /*force*/);
    } else {
      const geometry = this.geometry.clone();
      geometry.resize(this.mapView.getResolution(), originPoint);
      geometry.move(projectedXY[0], projectedXY[1]);

      this.point = new Feature(geometry);
      this.point.geometry.clearBounds();
      this.layer.addFeature(this.point);
    }
  }
}
