/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Shape} from '../../datasets/shape_dataset/shape.js';
import {Feature} from '../../syntool.js';
import {PointHandler} from './point_handler.js';

export class OLMushroomShapeHandler extends PointHandler {
  /*Public*/
  constructor(options) {
    super(options);
    this.type = Shape.MUSHROOM;
    this.shapeProps = options?.shapeProps || null;
    this.lastCenterXY = null;
  }

  activate(mapView) {
    if (!super.activate(mapView)) {
      return false;
    }
    this.listenToEvent(this.mapView.hub, 'change:zoom', this['change:zoom'], this);
    return true;
  }
  deactivate() {
    super.deactivate();
  }

  /*Event handlers*/
  'change:zoom'() {
    if (this.lastCenterXY) {
      this.modifyFeature(this.lastCenterXY);
    }
  }

  /*Protected*/
  getShape(projectedXY) {
    if (projectedXY) {
      this.lastCenterXY = projectedXY;
    } else {
      projectedXY = this.lastCenterXY;
    }

    const resolution = (
      this.mapView.convertToUnit(this.mapView.getResolution(), 'm')
    );
    const centerPos = 50 /*px*/ * resolution/*m/px*/;
    const radius    = 40 /*px*/ * resolution/*m/px*/;

    return Object.assign({
      baseAnchor : [0, 0.5],
      baseWidth  : 50  /*px*/ * resolution /*m/px*/,
      baseHeight : 140 /*px*/ * resolution /*m/px*/,
      baseAngle  : 0,
      leftLobeRx : radius,
      leftLobeRy : radius,
      rightLobeRx: radius,
      rightLobeRy: radius,
    }, this.shapeProps, {
      type: Shape.MUSHROOM,
      leftLobeCenter : [projectedXY[0] - centerPos, projectedXY[1] + centerPos],
      rightLobeCenter: [projectedXY[0] + centerPos, projectedXY[1] + centerPos],
    });
  }

  createGeometry(shape) {
    return Shape.createGeometry(shape);
  }

  modifyFeature(projectedXY) {
    const shape = this.getShape(projectedXY);
    const geometry = this.createGeometry(shape);

    if (this.point) {
      this.point.geometry = geometry;
      this.point.properties.shape = shape;
      this.layer.drawFeature(this.point, true /*force*/);
    } else {
      this.point = new Feature(geometry, {shape});
      this.layer.addFeature(this.point);
    }
  }
}
