/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Geometry, Feature} from '../../syntool.js';
import {PathHandler} from './path_handler.js';

export class PolygonHandler extends PathHandler {
  /*Public*/
  constructor(options) {
    super(options);
    this.polygon = null;
  }

  get canFinish() {
    return this.getCurrentPointIndex() > 2;
  }

  /*Protected*/
  getGeometry() {
    return this.polygon?.geometry;
  }

  modifyFeature(projectedXY) {
    if (!this.line) {
      const polygonGeometry = Geometry.createPolygonFromXYs([[projectedXY]]);
      const lineGeometry = Geometry.getComponentAtIndex(polygonGeometry, 0);
      const pointGeometry = Geometry.getComponentAtIndex(lineGeometry, 0);
      this.point = new Feature(pointGeometry);
      this.line = new Feature(lineGeometry);
      this.polygon = new Feature(polygonGeometry);
      this.layer.addFeatures([this.polygon, this.point]);
      return;
    }
    super.modifyFeature(projectedXY);
  }
  destroyFeature(force) {
    super.destroyFeature(force);
    this.polygon = null;
  }
}
