/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {HIDKeys, HIDModifiers, Syntool} from '../syntool.js';
import '../map_view_extensions/map_view_dataset_selection.js';
import '../datasets/shape_dataset/syntool_filters.js';

function isVisibleShape(dataset) {
  return (this.filters.dataset.isShape(dataset) && dataset.isVisible());
}
function isVisibleUserShape(dataset) {
  return (this.filters.dataset.isUserShape(dataset) && dataset.isVisible());
}

Syntool.extend({
  [Syntool.NAME]: 'ShapesShortcuts',
  [Syntool.INIT]() {
    this.shortcutsStore.add(
      'Global',
      [
        {
          id         : 'shapes-select-all-visible',
          modifiers  : [HIDModifiers.ctrlOrMeta],
          keys       : HIDKeys.A,
          description: 'Select all visible shapes',
          handler    : () => {
            this.mapView.selectDatasets(
              this.mapView.getDatasetsInMap(isVisibleShape, this)
            );
            return true;
          },
        },
        {
          id         : 'user-shapes-select-all-visible',
          modifiers  : [HIDModifiers.ctrlOrMeta, HIDModifiers.shift],
          keys       : HIDKeys.A,
          description: 'Select all visible user shapes',
          handler    : () => {
            this.mapView.selectDatasets(
              this.mapView.getDatasetsInMap(isVisibleUserShape, this)
            );
            return true;
          },
        },
      ]
    );

    const updateShapesSelectionShortcuts = () => {
      this.shortcutsStore.setShortcutsWithIdsEnabled([
        'shapes-select-all-visible',
        'user-shapes-select-all-visible',
      ], (
        this.mapView.datasetInteractionsActive &&
        this.mapView.datasetSelectionActive
      ));
    };
    this.mapView.hub
      .on('change:datasetSelectionActive', updateShapesSelectionShortcuts)
      .on('change:datasetInteractionsActive', updateShapesSelectionShortcuts);
    updateShapesSelectionShortcuts();
  },
});
