/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {HIDModifiers, HIDKeys, renderModifiedKey, Syntool} from '../syntool.js';
import {copyToClipboard} from '../utils/clipboard.js';

const copyKey = renderModifiedKey(HIDModifiers.ctrlOrMeta, HIDKeys.C);

Syntool.extend({
  [Syntool.NAME]: 'Clipboard',

  copyToClipboard(text, options) {
    const {label = 'value', onWaiting, onDone} = options;
    let messageId = null;

    copyToClipboard(text, () => {
      messageId = this.notificationCenter.showInfo(
        `Press ${copyKey} to copy ${label} to clipboard`,
        -1 /* keep it visible */
      );
      if (onWaiting) {
        onWaiting();
      }
      return true;
    }, (succeeded) => {
      if (messageId) {
        this.notificationCenter.closeMessage(messageId);
        messageId = null;
      }
      if (succeeded) {
        this.notificationCenter.showInfo(`Copied ${label} to clipboard`);
      }
      if (onDone) {
        onDone(succeeded);
      }
    });
  },
});
