/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {
  SECOND_IN_MILLISECONDS,
  throttleOncePerMs,
  updateUrlQueryString,
  Syntool,
} from '../syntool.js';
import {
  appStateToJSON,
  appStateJSONToURLParameters,
  appStateFromURL,
} from '../utils/app_state.js';
import {updateURLParams} from '../utils/url_utils.js';

export function getAppStateURLParameters() {
  return appStateJSONToURLParameters(appStateToJSON(Syntool.state));
}

var getStateURLParameters = getAppStateURLParameters;

export function getStateURLParametersGetter() {
  return getStateURLParameters;
}
export function setStateURLParametersGetter(getter) {
  getStateURLParameters = getter;
}

export function getStateURL() {
  return updateUrlQueryString(window.location, getStateURLParameters());
}

function updatePageURL() {
  updateURLParams(getStateURLParameters());
}
function savePageURLInSession() {
  const href = window.location.href;
  const newHref = getStateURL().href;
  if (href !== newHref) {
    try {
      sessionStorage.setItem('SyntoolURLState', JSON.stringify({href, newHref}));
      updatePageURL();
    } catch (error) {
      /*ignore*/
    }
  }
}
function getCurrentStateURL() {
  try {
    const savedState = sessionStorage.getItem('SyntoolURLState');
    if (savedState) {
      const {href, newHref} = JSON.parse(savedState);
      if (href === window.location.href) {
        return newHref;
      }
      sessionStorage.removeItem('SyntoolURLState');
    }
  } catch (error) {
    /*ignore*/
  }
  return window.location.href;
}

export function getStateFromURL() {
  return appStateFromURL(getCurrentStateURL());
}

Syntool.whenInitialised(function initURLStateUpdater() {
  const handleAppStateChange = (
    throttleOncePerMs(SECOND_IN_MILLISECONDS, updatePageURL)
  );
  Syntool.hub.on('change:state', handleAppStateChange);
  handleAppStateChange();

  document.addEventListener('visibilitychange', () => {
    if (document.visibilityState === 'hidden') {
      savePageURLInSession();
    }
  });
  window.addEventListener('pagehide', savePageURLInSession);
});

window.ODL = Object.assign(window.ODL || {}, {
  getAppStateURLParameters,
  getStateURLParametersGetter,
  setStateURLParametersGetter,
  getStateURL,
});
