/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {zerosArray, getNowMs} from '../syntool.js';

export class CircularBuffer {
  constructor(capacity, age) {
    this._buffer = zerosArray(capacity);
    this._head = 0;
    this.age = age;
    this.size = 0;
  }

  push(value) {
    const now = getNowMs();
    const capacity = this._buffer.length;

    this.cleanup(now);

    // If full, remove last entry
    if (this.size === capacity) {
      this._head = (this._head + 1) % capacity;
      this.size--;
    }

    // Add new entry
    const tail = (this._head + this.size) % capacity;
    this._buffer[tail] = {value: value, death: now + this.age};
    this.size++;
  }
  getHead() {
    this.cleanup();
    return this.size === 0 ? null : this._buffer[this._head].value;
  }
  getTail() {
    this.cleanup();
    const capacity = this._buffer.length;
    const tail = (this._head + this.size - 1) % capacity;
    return this.size === 0 ? null : this._buffer[tail].value;
  }
  cleanup(now = getNowMs()) {
    const capacity = this._buffer.length;

    while (this.size > 0 && this._buffer[this._head].death < now) {
      this._head = (this._head + 1) % capacity;
      this.size--;
    }
  }
  clear() {
    this._head = 0;
    this.size = 0;
  }
}
window.ODL = Object.assign(window.ODL || {}, {CircularBuffer});
