/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {
  emptyFunction,
  extendClass,
  HIDKeys,
  eventMatchesKey,
  MapView,
} from '../syntool.js';
import './map_view_dataset_interactions.js';
import './map_view_dataset_selection.js';

extendClass(MapView, [
  /*Public - Dataset picking*/ {
    pickDataset() {
      return new Promise(this._startPicking.bind(this));
    },
    abortPicking: emptyFunction,
  },
  /*Protected - Dataset picking*/ {
    _startPicking(resolve) {
      const selectionActiveBeforePicking = this._datasetSelectionActive;
      const selectedDatasetsBeforePicking = this.getSelectedDatasets();
      this.deactivateDatasetSelection();

      const finishPicking = (dataset) => {
        this._deselectAllDatasets();
        this.abortPicking = emptyFunction;

        this.syntool.keyboardManager.hub
          .off('change:up', this.handlePickingKeyUp, this);
        this.hub
          .off('dataset:hover', this.handlePickingDatasetHover, this)
          .off('dataset:unhover', this.handlePickingDatasetUnhover, this)
          .off('dataset:click', finishPicking)
          .emit('picking:end', dataset);

        if (selectionActiveBeforePicking) {
          this.activateDatasetSelection();
        }
        this.selectDatasets(selectedDatasetsBeforePicking);

        resolve(dataset);
      };

      this.abortPicking = () => finishPicking(null);

      this.syntool.keyboardManager.hub
        .on('change:up', this.handlePickingKeyUp, this);
      this.hub
        .on('dataset:hover', this.handlePickingDatasetHover, this)
        .on('dataset:unhover', this.handlePickingDatasetUnhover, this)
        .on('dataset:click', finishPicking)
        .emit('picking:start');
    },
    handlePickingKeyUp(event) {
      if (event && eventMatchesKey(event, HIDKeys.escape)) {
        this.abortPicking();
      }
    },
    handlePickingDatasetHover(dataset) {
      this._selectDataset(dataset);
    },
    handlePickingDatasetUnhover(dataset) {
      this._deselectDataset(dataset);
    },
  },
]);
