/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {extendClass, Dataset, MapView} from '../syntool.js';

extendClass(MapView, [
  /*Protected - Extensions*/ {
    initialize(props) {
      window.original();
      this._datasetsResources = new Map();
    },
  },
  /*Public - Dataset resources*/ {
    hasResourcesForDataset(dataset) {
      return this._datasetsResources.has(dataset.datasetId);
    },
    destroyResourcesForDataset(dataset) {
      this._datasetsResources.delete(dataset.datasetId);
    },

    hasResourceForDataset(dataset, resourceKey) {
      return (
        this.hasResourcesForDataset(dataset) &&
        Object.hasOwn(this.getResourcesForDataset(dataset), resourceKey)
      );
    },
    getResourceForDataset(dataset, resourceKey, defaultValue) {
      if (this.hasResourcesForDataset(dataset)) {
        return this.getResourcesForDataset(dataset)[resourceKey];
      }
      if (defaultValue !== undefined) {
        this.setResourceForDataset(dataset, resourceKey, defaultValue);
      }
      return defaultValue;
    },
    setResourceForDataset(dataset, resourceKey, value) {
      var resources = this.getResourcesForDataset(dataset) || {};
      resources[resourceKey] = value;
      if (!this.hasResourcesForDataset(dataset)) {
        this.setResourcesForDataset(dataset, resources);
      }
    },
    destroyResourceForDataset(dataset, resourceKey) {
      if (this.hasResourcesForDataset(dataset)) {
        delete this.getResourcesForDataset(dataset)[resourceKey];
      }
    },
  },
  /*Protected - Dataset resources*/ {
    getResourcesForDataset(dataset) {
      return this._datasetsResources.get(dataset.datasetId);
    },
    setResourcesForDataset(dataset, resources) {
      return this._datasetsResources.set(dataset.datasetId, resources);
    },
  },
]);
extendClass(Dataset, {
  hasResources() {
    return !!this.syntool.mapView.hasResourcesForDataset(this);
  },
  destroy() {
    this.syntool.mapView.destroyResourcesForDataset(this);
  },
});
