/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {extendClass, MapView} from '../syntool.js';

extendClass(MapView, [
  /*Protected - Extensions*/ {
    initialize() {
      window.original();

      this._lastMouseMoveEvent = null;
      this.hub.on('mouseleave+passive+capture', this.handleMousePositionLeave, this);
      this.hub.on('mousemove+passive+capture', this.updateMousePositionFromMouseEvent, this);
    },
  },
  /*Protected - Mouse position event handlers*/ {
    handleMousePositionLeave(event) {
      if (event.target === this.node) {
        this._lastMouseMoveEvent = null;
        this.hub.emit('change:mouse-position', null);
      }
    },
    updateMousePositionFromMouseEvent(event) {
      this._lastMouseMoveEvent = event;
      this.hub.emit(
        'change:mouse-position',
        this.getProjectedXYFromMouseEvent(event)
      );
    },
  },
  /*Public*/ {
    get lastMouseMoveEvent() {
      return this._lastMouseMoveEvent;
    },
    get lastMouseProjectedXY() {
      return (
        this._lastMouseMoveEvent &&
        this.getProjectedXYFromMouseEvent(this._lastMouseMoveEvent)
      );
    },
  },
]);
