/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {extendClass, MapView} from '../syntool.js';
import {supportsPixelated} from '../utils/dom_utils.js';

extendClass(MapView, [
  /*Protected - Extensions*/ {
    initialize() {
      window.original();

      this._tilesPixelInterpolationActive = !supportsPixelated;

      this._stateHooks.push({
        get: ({tilesPixelInterpolationActive}) => (
          {tilesPixelInterpolationActive}
        ),
        set: ({tilesPixelInterpolationActive}) => {
          this.tilesPixelInterpolationActive = tilesPixelInterpolationActive;
        },
      });
    },
  },
  /*Public*/ {
    get tilesPixelInterpolationSupported() {
      return supportsPixelated;
    },
    get tilesPixelInterpolationActive() {
      return this._tilesPixelInterpolationActive;
    },
    set tilesPixelInterpolationActive(active) {
      if (supportsPixelated && this._tilesPixelInterpolationActive === !active) {
        this._tilesPixelInterpolationActive = !!active;
        this.node.classList.toggle('syntool-tile-interpolation', !!active);
        this._invalidateState();
      }
    },
  },
]);
