/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Z_INDEX, extendClass, MapView} from '../syntool.js';
import {shapeStyleForIntent} from './map_view_shapes.js';

extendClass(MapView, [
  /*Protected - Extensions*/ {
    initialize() {
      window.original();

      let defaultStyle = null;
      const shapesDrawingLayer = new this.shapesLayer.constructor(
        'Shapes Drawing',
        {
          style: {
            base: ({properties: {shape}}) => {
              if (!shape && defaultStyle) {
                return defaultStyle;
              }
              const style = Object.assign(
                {},
                shapeStyleForIntent(
                  this.syntool.userShapesProduct,
                  shape,
                  'base'
                ),
                shapeStyleForIntent(
                  this.syntool.userShapesProduct,
                  shape,
                  'temporary'
                )
              );
              if (!shape) {
                defaultStyle = style;
              }
              return style;
            }
          },
        }
      );

      this.shapesDrawingLayer = shapesDrawingLayer;
      this.addLayer(shapesDrawingLayer);
    },
  },
  /*Protected - Layer Z-Index*/ {
    getZIndexForLayer(layer, index) {
      if (layer === this.shapesDrawingLayer) {
        return Z_INDEX.DRAWING_HANDLERS;
      }
      return window.original();
    },
  },
]);
