/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Z_INDEX, extendClass, MapView} from '../syntool.js';
import {OLModifyShape} from './shapes/ol_modify_shape_control.js';
import './map_view_shapes.js';

extendClass(MapView, [
  /*Protected - Extensions*/ {
    initialize() {
      window.original();

      var shapesEditingLayer = new this.shapesLayer.constructor(
        'Shapes Editing',
        {
          style: (feature, intent) => {
            const style = this.shapesLayer.style(feature, intent);
            if (intent === 'base') {
              /**
               * Prevent macOS's data detector from triggering unexpectedly.
               *
               * Example when a mushroom annotation is selected and a user
               * strong presses on a control point, macOS detects the numbers in
               * the control points and suggests adding a new contact.
               */
              style.labelSelect = false;
            }
            return style;
          },
        }
      );

      this.shapesEditingLayer = shapesEditingLayer;
      this.shapesEditingControl = new OLModifyShape(
        this,
        shapesEditingLayer,
        {vertexRenderIntent: 'edit'}
      );
      this.addLayer(shapesEditingLayer);
    },
  },
  /*Protected - Layer Z-Index*/ {
    getZIndexForLayer(layer, index) {
      if (layer === this.shapesEditingLayer) {
        return Z_INDEX.EDITING_HANDLERS;
      }
      return window.original();
    },
  },
]);
