/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
/*eslint-env worker*/
importScripts('thirdparty/zlib.js');
importScripts('thirdparty/png.lite.js');

var CHANNEL_NAMES = ['modulus', 'angle', 'color'];

self.onmessage = function(event) {
  try {
    var result = decodePNG(event.data[1]);
    postMessage([event.data[0], 0, result], [result.pixels.buffer]);
  } catch (error) {
    postMessage([event.data[0], 1, error.toString()]);
  }
};

function metadataFromPNG(data) {
  var channels = data.colors + (data.hasAlphaChannel ? 1 : 0);
  var offsets = new Float32Array(channels);
  var scales = new Float32Array(channels);

  CHANNEL_NAMES.forEach(function(name) {
    // -1 because Geotiff array indexing begins at 1
    var channel = parseInt(data.text[name + '_channel'], 10) - 1;
    if (!isNaN(channel)) {
      offsets[channel] = parseFloat(data.text[name + '_offset']);
      scales[channel] = parseFloat(data.text[name + '_scale']);
    }
  });

  return {
    width      : data.width,
    height     : data.height,
    channels   : channels,
    offsets    : offsets,
    scales     : scales,
    nodataValue: parseInt(data.text.nodata_value, 10),
  };
}
function decodePNG(rawPNG) {
  var data = new Uint8Array(rawPNG);
  var png = new self.PNG(data);
  var pixels = png.decodePixels(data.imgData);

  return {
    metadata: metadataFromPNG(png),
    pixels  : pixels,
  };
}
