/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Geometry, Types} from '../syntool.js';
import {Shape} from '../datasets/shape_dataset/shape.js';

var utils = Shape.utils;

Shape.addShapeType('TEXT', {
  typeChecker: Types.shape({
    text    : Types.string.match(/\S+/).isRequired,
    location: Types.Point.isRequired,
  }),
  project(shape) {
    shape.location = utils.projectedPoint(shape.location);
  },
  unproject(shape) {
    shape.location = utils.unprojectedPoint(shape.location);
  },
  create(shape, props) {
    shape.location = utils.correctedPoint(props.location);
  },
  same(a, b) {
    return utils.samePoint(a.location, b.location);
  },
  createGeometry(shape) {
    return Geometry.createVirtualPointFromXY(shape.location);
  },

  changesOnControlPointMove(shape, key, x, y) {
    return {location: [x, y]};
  },
  updatesFromChanges(shape, changes) {
    return changes;
  },
});
