/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {linkStyleSheet} from '../utils/dom_utils.js';
import {H, DOMEvents} from '../syntool.js';
import {FlagsStore} from '../features/flags_store.js';
import {Dialog} from './components/dialog.js';
import {opensInNewTabIcon} from './utils.js';

linkStyleSheet('css/about.css');

export class AboutDialog extends Dialog {
  constructor(id, props, options) {
    if (!props && typeof id !== 'string') {
      props = id;
      id = undefined;
    }

    const {content, storageShowKey, onBeforeShow, onShow, ...otherProps} = props;
    let dontShowCheckbox;
    super(id, {
      autoLocationReset: false,
      content: H([
        content,
        H('div', {class: 'action-buttons'}, [
          H('label', [
            dontShowCheckbox = H('input', {
              type: 'checkbox',
              class: 'js-dont-show-again',
            }),
            "Don't show again",
          ]),
          H('button', {class: 'startTourButton js-close-button js-start'},
            'Start Quick Tour'
          ),
        ]),
      ]),
      onBeforeShow(dialog) {
        dontShowCheckbox.checked = FlagsStore.get(storageShowKey) !== true;
        dialog.resetLocation();
        if (typeof onBeforeShow === 'function') {
          onBeforeShow.call(props.context, dialog);
        }
      },
      onShow(dialog) {
        dialog.contentNode.firstChild.scrollTop = 0;
        if (typeof onShow === 'function') {
          onShow.call(props.context, dialog);
        }
      },
      onHide() {
        FlagsStore.set(storageShowKey, !dontShowCheckbox.checked);
      },
      ...otherProps,
    }, options);

    this.storageShowKey = storageShowKey;

    content?.classList.add('-with-scroll-shadows');
    for (const anchor of content.querySelectorAll('a[target="_blank"]')) {
      anchor.setAttribute('rel', 'noopener');
      anchor.insertAdjacentHTML('beforeend', opensInNewTabIcon);
    }

    DOMEvents(this.node)
    .on('click', '.js-start', () => {
      this.syntool.controls.tour?.start();
    })
    .on('change', '.js-dont-show-again', (event) => {
      FlagsStore.set(storageShowKey, !event.target.checked);
    });
  }

  showIfNeeded(props) {
    if (FlagsStore.get(this.storageShowKey) !== false) {
      this.show(props);
    }
  }
}
