/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {throttleOncePerAnimationFrame, Control, h} from '../../syntool.js';
import {linkStyleSheet} from '../../utils/dom_utils.js';

linkStyleSheet('css/attribution_control.css');

const PARTS_SEPARATOR = /\s*\|\|\s*/g;

export class AttributionControl extends Control {
  constructor(options) {
    super(options);

    this.node = document.createElement('div');
    this.node.className = 'AttributionControl';

    this.render = throttleOncePerAnimationFrame(this.render);
  }

  onActivate() {
    this.syntool.usableViewportNode.appendChild(this.node);
    this.syntool.mapView.hub
      .on('layer:did-add-to-map', this.render, this)
      .on('layer:did-remove-from-map', this.render, this)
      .on('layer:change:visibility', this.render, this)
      .on('layer:change:attribution', this.render, this);
    this.render();
  }
  onDeactivate() {
    this.syntool.mapView.hub
      .off('layer:did-add-to-map', this.render, this)
      .off('layer:did-remove-from-map', this.render, this)
      .off('layer:change:visibility', this.render, this)
      .off('layer:change:attribution', this.render, this);
    this.syntool.usableViewportNode.removeChild(this.node);
  }

  render() {
    const {mapView} = this.syntool;
    const attributions = new Set();
    const parts = new Set();
    for (const layer of mapView.getLayers()) {
      const {attribution} = layer;
      if (attribution && layer.getVisibility()) {
        const [attr, ...otherParts] = attribution.split(PARTS_SEPARATOR);
        attributions.add(attr);
        for (const part of otherParts) {
          parts.add(part);
        }
      }
    }
    this.node.innerHTML = h(
      [[...attributions].join(' | '), ...parts].map((part) => h('span', part))
    );
  }
}
window.ODL = Object.assign(window.ODL || {}, {AttributionControl});
