/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {H, MapView} from '../../syntool.js';

export function createBackgroundLayerPicker(baseLayers, selectedLayerName, onChange) {
  const select = H('select', {title: 'Base map'}, baseLayers.map((layer) => {
    const layerName = MapView.getLayerName(layer);
    return H('option', {value: layerName}, layerName);
  }));
  select.onchange = (event) => {
    onChange(event.target.value);
  };

  function setLayerName(layerName) {
    for (const option of select.options) {
      option.textContent = option.value;
    }
    select.value = layerName;
    for (const option of select.selectedOptions) {
      option.textContent += ' ✓';
    }
  }

  setLayerName(selectedLayerName);

  return {node: select, setLayerName};
}

window.ODL = Object.assign(window.ODL || {}, {createBackgroundLayerPicker});
