/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {
  formatUTCDate,
  throttleOncePerAnimationFrame,
  Control,
} from '../../syntool.js';
import {linkStyleSheet} from '../../utils/dom_utils.js';

linkStyleSheet('css/date_time_control.css');

export class DateTimeControl extends Control {
  constructor(options) {
    super(options);

    this.node = document.createElement('div');
    this.node.className = 'DateTimeControl';

    this.render = throttleOncePerAnimationFrame(this.render);
  }

  onActivate() {
    this.syntool.uiOverlayNode.appendChild(this.node);
    this.syntool.timelineStore.hub.on('change:currentDate', this.render, this);
    this.render();
  }
  onDeactivate() {
    this.syntool.uiOverlayNode.removeChild(this.node);
    this.syntool.timelineStore.hub.off('change:currentDate', this.render, this);
  }

  render() {
    this.node.textContent = formatUTCDate(
      this.syntool.timelineStore.currentDate,
      '$<date>\n$<time> UTC'
    );
  }
}
window.ODL = Object.assign(window.ODL || {}, {DateTimeControl});
