/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {ajax, ajaxRetriable, h} from '../../syntool.js';
import {linkStyleSheet} from '../../utils/dom_utils.js';
import {marked} from '../../vendors/marked.js';
import {Dialog} from './dialog.js';

linkStyleSheet('css/markdown_dialog.css');

export class MarkdownDialog extends Dialog {
  /*Public*/
  constructor(id, props, options) {
    super(id, props, options);
    this.node.classList.add('MarkdownDialog');
  }

  show(props) {
    this.setProps(props);

    var url = this.props.contentURL;
    var onError = this.props.onError;
    super.show({
      content(node) {
        ajaxRetriable(ajax.get)(url)
          .then(marked)
          .catch(function(error) {
            if (typeof onError === 'function') {
              return onError.call(this, error, node);
            }
            console.error(
              onError.console || 'Error loading markdown content at ' + url,
              error
            );
            return h([
              h('h1', {style: 'color: red'},
                onError.header || 'Error loading contents'
              ),
              h('details', [
                h('summary', null, error),
                h('pre', null, error.stack),
              ]),
            ]);
          })
          .then(function(html) {
            node.innerHTML = html;
            node.scrollTop = 0;
            node.scrollLeft = 0;
          });
      },
    });
  }
}
window.ODL = Object.assign(window.ODL || {}, {MarkdownDialog});
