/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {H, DOMEvents, Control} from '../../syntool.js';
import {linkStyleSheet} from '../../utils/dom_utils.js';
import {SelectionInfoView} from '../selection_panel/selection_info_view.js';

const cssLoadPromise = linkStyleSheet('css/selection_info_panel.css');

export class SelectionInfoPanel extends Control {
  /*Public*/
  constructor(options) {
    super(options);

    this.view = new SelectionInfoView();
    this.view.hub.on('render', this.handleRender, this);

    let lid;
    this.node = H('div', {
      class: 'UISidebar SelectionInfoPanel',
      style: 'display: none',
    }, [
      lid = H('div', {class: 'lid horizontal-lid h-hidden-in-screenshot-mode'}),
      this.view.node,
    ]);

    DOMEvents(lid).on('click', () => this.toggle());
    DOMEvents(this.view.node).on('focus', () => (this.open = true));
  }

  get state() {
    return Object.assign(super.state, {open: this.open});
  }
  set state(state) {
    super.state = state;
    this.open = state.open;
  }

  get open() {
    return !this.node.classList.contains('-closed');
  }
  set open(open) {
    if (this.open === !open) {
      this.node.classList.toggle('-closed', !open);
      if (open) {
        this.node.classList.remove('-has-new');
      }
      this.hub.emit('change:open');
      this.hub.emit('change:state');
    }
  }
  toggle(open = !this.open) {
    this.open = open;
  }

  /*Protected*/
  async onActivate() {
    const {syntool} = this;

    await cssLoadPromise;
    if (this.syntool !== syntool) {
      return;
    }

    this.syntool.usableViewportNode.appendChild(this.node);
    this.syntool.mapView.hub
      .on('change:selection', this.handleSelectionChange, this);
    this.handleSelectionChange();
  }
  onDeactivate() {
    if (!this.node.parentNode) {
      return;
    }

    this.syntool.usableViewportNode.removeChild(this.node);
    this.syntool.mapView.hub
      .off('change:selection', this.handleSelectionChange, this);
    this.view.setDatasets([]);
  }

  handleRender() {
    this.node.style.display = (this.view.isEmpty ? 'none' : '');
    this.node.classList.toggle('-has-new', !this.open && this.view.shouldNotify);
  }
  handleSelectionChange() {
    this.view.setDatasets(this.syntool.mapView.getSelectedDatasets());
  }
}
window.ODL = Object.assign(window.ODL || {}, {SelectionInfoPanel});
