/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {H, DOMEvents, Control} from '../../syntool.js';
import {linkStyleSheet} from '../../utils/dom_utils.js';

linkStyleSheet('css/zoom_control.css');

export class ZoomControl extends Control {
  /*Public*/
  constructor(options) {
    super(options);

    this.node = H('div', {id: 'zoom-controls'}, [
      H('button', {class: 'js-zoom-in'},
        H('span', {class: '-icon -icon-plus'})
      ),
      H('button', {class: 'js-zoom-out'},
        H('span', {class: '-icon -icon-minus'})
      ),
    ]);

    DOMEvents(this.node)
    .on('click', '.js-zoom-in', (event) => {
      event.preventDefault();
      this.syntool.mapView.zoomIn();
    })
    .on('click', '.js-zoom-out', (event) => {
      event.preventDefault();
      this.syntool.mapView.zoomOut();
    });
  }

  onActivate() {
    this.syntool.uiOverlayNode.appendChild(this.node);
  }
  onDeactivate() {
    this.syntool.uiOverlayNode.removeChild(this.node);
  }
}
window.ODL = Object.assign(window.ODL || {}, {ZoomControl});
