/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {HIDKeys, HIDModifiers, h, renderModifiedKey} from '../../syntool.js';
import {FlagsStore} from '../../features/flags_store.js';
import {linkStyleSheet} from '../../utils/dom_utils.js';
import {Dialog} from '../components/dialog.js';

linkStyleSheet('css/pagination_help.css');

export class PaginationHelpDialog extends Dialog {
  constructor(timelinePanel) {
    super('pagination-help', {
      title: 'Pagination controls',
      arrow: 'bottom',
      content: h([
        h('p', [
          "When there's too much datasets (more than 100 by default), ",
          'they get split into pages (of size 100 by default).',
        ]),
        h('p', [
          'The number of currently displayed datasets is shown between the ',
          'two buttons.',
        ]),
        h('p', [
          'You can use the left and right buttons to change the displayed ',
          'page.',
        ]),
        h('p', [
          'You can also use the hotkeys ',
          renderModifiedKey(HIDModifiers.shift, HIDKeys.B),
          ' and ',
          renderModifiedKey(HIDModifiers.shift, HIDKeys.N),
          '.',
        ]),
        h('p',
          h('em',
            'You can click on the displayed count to show this message again.'
          )
        ),
      ]),
    });

    this.timelinePanel = timelinePanel;
  }

  toggle() {
    if (!this.active) {
      this.timelinePanel.syntool?.addControl(this);
    }
    super.toggle();
  }
  show() {
    if (window.innerWidth < 500 || window.innerHeight < 600) {
      return;
    }

    if (this.syntool !== this.timelinePanel.syntool) {
      if (this.timelinePanel.syntool) {
        this.timelinePanel.syntool.addControl(this);
      } else {
        this.syntool.removeControl(this);
      }
    }
    if (!this.active) {
      return;
    }

    var prevPageButton = this.timelinePanel.node.querySelector('.js-prev-page');
    var nextPageButton = this.timelinePanel.node.querySelector('.js-next-page');
    var prevPageButtonRect = prevPageButton.getBoundingClientRect();
    var nextPageButtonRect = nextPageButton.getBoundingClientRect();
    var centerX = (prevPageButtonRect.left + nextPageButtonRect.right) / 2;

    this.resetLocation();
    this.node.style.left = centerX + 'px';

    // Move the pagination help popup into the TimelinePanel so that it moves
    // with it
    this.timelinePanel.node.appendChild(this.node);

    super.show();
    FlagsStore.set('paginationHelpShown', true);
  }
  showIfNeeded() {
    if (!FlagsStore.get('paginationHelpShown')) {
      this.show();
    }
  }
}
