/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
/*global SimileAjax: false*/

/*==================================================
 *  Custom Date Labeller
 *==================================================
 */

const MONTH_NAMES = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];

export class TimelineCustomDateLabeller {
  labelInterval(date, intervalUnit) {
    let text;
    let emphasized = false;
    let v;

    switch (intervalUnit) {
    case SimileAjax.DateTime.MILLISECOND:
      v = date.getUTCMilliseconds();
      if (v === 0) {
        emphasized = true;
      } else {
        text = v;
        break;
      }
      /*fallthrough*/
    case SimileAjax.DateTime.SECOND:
      v = date.getUTCSeconds();
      if (v === 0) {
        text = this.labelInterval(date, intervalUnit + 1).text + ':00';
        emphasized = true;
      } else {
        text = v + 's';
      }
      break;
    case SimileAjax.DateTime.MINUTE:
      v = date.getUTCMinutes();
      if (v === 0) {
        emphasized = true;
      } else {
        text = v + 'mn';
        break;
      }
      /*fallthrough*/
    case SimileAjax.DateTime.HOUR:
      v = date.getUTCHours();
      if (v === 0) {
        emphasized = true;
      } else {
        text = v + 'hr';
        break;
      }
      /*fallthrough*/
    case SimileAjax.DateTime.DAY:
    case SimileAjax.DateTime.WEEK:
      if (date.getUTCMonth() === 0 && date.getUTCDate() === 1) {
        emphasized = true;
      } else {
        text = MONTH_NAMES[date.getUTCMonth()] + ' ' + date.getUTCDate();
        break;
      }
      /*fallthrough*/
    case SimileAjax.DateTime.MONTH:
      v = date.getUTCMonth();
      if (v !== 0) {
        text = MONTH_NAMES[v];
        break;
      }
      /*fallthrough*/
    case SimileAjax.DateTime.YEAR:
    case SimileAjax.DateTime.DECADE:
    case SimileAjax.DateTime.CENTURY:
    case SimileAjax.DateTime.MILLENNIUM:
      v = date.getUTCFullYear();
      if (v > 0) {
        text = date.getUTCFullYear();
      } else {
        text = (1 - v) + 'BC';
      }
      emphasized = (
        (intervalUnit < SimileAjax.DateTime.YEAR) ||
        (intervalUnit === SimileAjax.DateTime.DECADE && v % 100 === 0) ||
        (intervalUnit === SimileAjax.DateTime.CENTURY && v % 1000 === 0)
      );
      break;
    default:
      text = date.toUTCString();
    }
    return {text, emphasized};
  }
}
