/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {getNowMs, DOMEvents} from '../syntool.js';
import * as Gestures from './utils/gestures.js';

export * from './utils/clear_text_field_icon.js';
export * from './utils/opens_in_new_tab_icon.js';
export * from './utils/show_confirm.js';
export * from './utils/show_permalink.js';
export {Gestures};

export function preventFocusOnClick(element, selector = null) {
  let ignoreMouseDown = false;
  let ignoreMouseUp = false;
  let lastFocusedElement = null;
  let focusInDeadline = 0;

  DOMEvents(element)
  .on(['mousedown', 'touchstart'], selector, (event) => {
    if (event.type === 'touchstart') {
      ignoreMouseDown = true;
    } else if (ignoreMouseDown) {
      ignoreMouseDown = false;
      return;
    }
    lastFocusedElement = document.activeElement;
    focusInDeadline = getNowMs() + 100;
  })
  .on(['mouseup', 'touchend'], selector, (event) => {
    if (event.type === 'touchend') {
      ignoreMouseUp = true;
    } else if (ignoreMouseUp) {
      ignoreMouseUp = false;
      return;
    }
    focusInDeadline = getNowMs() + 100;
  })
  .on('focusin', selector, (event) => {
    if (focusInDeadline > getNowMs() && lastFocusedElement !== event.target) {
      event.target.blur();
      lastFocusedElement?.focus();
    }
  });
}
