/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {h, css, DOMEvents} from '../../syntool.js';
import {Dialog} from '../components/dialog.js';

document.head.appendChild(css`
  #confirm-dialog .Dialog-Content {
    padding: 15px;
    contain: content;
  }
  #confirm-dialog .Dialog-Content :first-child {
    margin-top: 0;
  }
  #confirm-dialog .Dialog-Content .confirm-buttons {
    display: flex;
    justify-content: space-between;
  }
`);

var confirmDialog = null;
var confirmResolve;
var confirmReject;

function getDialog(syntool) {
  if (!confirmDialog) {
    confirmDialog = new Dialog('confirm-dialog', {closable: false});
    confirmDialog.hub.on('hide', function() {
      if (confirmReject) {
        confirmReject();
      }
      if (confirmDialog.active) {
        confirmDialog.syntool.removeControl(confirmDialog);
      }
    });
    DOMEvents(confirmDialog.node)
      .on('click', '.confirm-no', function() {
        if (confirmResolve) {
          confirmResolve(false);
        }
        confirmDialog.hide();
      })
      .on('click', '.confirm-yes', function() {
        if (confirmResolve) {
          confirmResolve(true);
        }
        confirmDialog.hide();
      });
  }
  syntool?.addControl(confirmDialog);
  return confirmDialog;
}

export function showConfirm(options) {
  options = Object.assign({
    title: 'Confirm',
    no   : 'No',
    yes  : 'Yes',
  }, options);

  if (confirmReject) {
    confirmReject();
  }
  return new Promise(function(resolve, reject) {
    confirmResolve = resolve;
    confirmReject = reject;
    getDialog(options.syntool).show({
      title  : options.title,
      content: renderConfirmContent(options),
    });
  });
}
function renderConfirmContent(options) {
  return [
    options.message && h('h3', {class: 'confirm-message'}, options.message),
    options.details && h('p', {class: 'confirm-details'}, options.details),
    h('div', {class: 'confirm-buttons'}, [
      h('button', {class: 'confirm-no', type : 'button'}, options.no),
      h('button', {class: 'confirm-yes', type : 'button'}, options.yes),
    ]),
  ].filter(Boolean).join('');
}
