/**
 * @author: <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {h, css, shortenURL} from '../../syntool.js';
import {Dialog} from '../components/dialog.js';

document.head.appendChild(css`
  #permalink-dialog .Dialog-Content {
    padding: 10px 20px;
    background: #000;
    display: flex;
    flex-direction: column;
  }
`);

var permalinkDialog = null;

function getDialog() {
  if (!permalinkDialog) {
    permalinkDialog = new Dialog('permalink-dialog', {
      title  : 'Permalink',
      content: h('input', {
        type    : 'text',
        readonly: true,
        value   : '',
      }),
    });
    permalinkDialog.hub.on('hide', function() {
      if (permalinkDialog.active) {
        permalinkDialog.syntool.removeControl(permalinkDialog);
      }
    });
  }
  return permalinkDialog;
}

export function showPermalink(url, syntool) {
  var dialog = getDialog();
  var input = dialog.contentNode.querySelector('input');
  input.value = 'creating...';
  input.disabled = true;

  syntool?.addControl(dialog);
  dialog.show();
  input.focus();
  input.select();

  shortenURL(url)
  .then(function(shortURL) {
    input.value = shortURL;
    input.disabled = false;
    input.focus();
    input.select();
  }, function(error) {
    console.error('Error shortening URL "' + url + '"!', error);
    syntool?.notificationCenter.showError(
      'Could not shorten URL due to server or network errors!'
    );
  });
}
