/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {NONPASSIVE, preventDefaultAndStopEvent} from '../syntool.js';
import {linkStyleSheet} from './dom_utils.js';

linkStyleSheet('css/global_drop_zone.css');

export function initGlobalDropZone(element, onDrop, canAccept) {
  var inCount = 0;
  var dropZoneElement = element;

  if (!element) {
    throw new Error('initGlobalDropZone::Missing required `element` argument');
  }

  document.addEventListener('dragenter', function(event) {
    preventDefaultAndStopEvent(event);

    if (inCount === 0) {
      if (!canAccept || canAccept(event)) {
        dropZoneElement.classList.add('is-over');
      }
    }
    inCount++;
  }, NONPASSIVE);
  document.addEventListener('dragover', preventDefaultAndStopEvent, NONPASSIVE);
  document.addEventListener('dragleave', function(event) {
    preventDefaultAndStopEvent(event);

    inCount--;
    if (inCount === 0) {
      dropZoneElement.classList.remove('is-over');
    }
  }, NONPASSIVE);
  document.addEventListener('drop', function(event) {
    preventDefaultAndStopEvent(event);

    inCount = 0;
    dropZoneElement.classList.remove('is-over');

    if (onDrop && !canAccept || canAccept(event)) {
      onDrop(event);
    }
  }, false);
}
window.ODL = Object.assign(window.ODL || {}, {initGlobalDropZone});
