/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {
  emptyFunction,
  emptyFunctionReturningNull,
} from '../syntool.js';

function setFeatureVisibility(feature, visible) {
  if (feature && (feature.style?.display !== 'none') !== visible) {
    if (visible) {
      feature.style.display = '';
    } else if (feature.style) {
      feature.style.display = 'none';
    } else {
      feature.style = {display: 'none'};
    }
    feature.layer?.drawFeature(feature);
    return true;
  }
  return false;
}

function getLayerOptions() {
  return {
    alwaysInRange          : true,
    isBaseLayer            : false,
    displayOutsideMaxExtent: false,
    wrapDateLine           : false,
  };
}
function getGridLayerOptions() {
  return Object.assign(getLayerOptions(), {
    /* Options */
    transitionEffect     : 'resize',
    removeBackBufferDelay: 0,

    /* Overrides */
    createBackBuffer: emptyFunctionReturningNull,
    queueTileDraw   : emptyFunction,
  });
}

export {
  setFeatureVisibility,
  getLayerOptions,
  getGridLayerOptions,
};
window.ODL = Object.assign(window.ODL || {}, {
  OLUtils: {
    setFeatureVisibility,
    getLayerOptions,
    getGridLayerOptions,
  },
});
