/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {updateUrlQueryString} from '../syntool.js';

const KEY_RE = /[%&#=+]+/g;
const VALUE_RE = /[%&#+]+/g;
const SPACE_RE = / /g;

function serializeSearchParam([key, value]) {
  return (
    key.replace(KEY_RE, encodeURIComponent) + '=' +
    value.replace(VALUE_RE, encodeURIComponent)
  );
}

function updateURLParams(paramsPartial) {
  const url = updateUrlQueryString(window.location, paramsPartial);
  url.search = (
    [...url.searchParams.entries()]
    .map(serializeSearchParam)
    .join('&')
    .replace(SPACE_RE, '+')
  );
  if (url.href !== window.location.href) {
    window.history.replaceState(null, null, url);
  }
}

export {updateURLParams};
window.ODL = Object.assign(window.ODL || {}, {updateURLParams});
