/*
 * From: https://github.com/simile-widgets/ancient-simile-widgets
 *
 *  (c) Copyright The SIMILE Project 2006. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
import {SimileAjax} from './init.js';

/*==================================================
 *  General, miscellaneous SimileAjax stuff
 *==================================================
 */

SimileAjax.ListenerQueue = function(wildcardHandlerName) {
  this._listeners = [];
  this._wildcardHandlerName = wildcardHandlerName;
};

SimileAjax.ListenerQueue.prototype.add = function(listener) {
  this._listeners.push(listener);
};

SimileAjax.ListenerQueue.prototype.remove = function(listener) {
  var listeners = this._listeners;
  for (var i = 0; i < listeners.length; i++) {
    if (listeners[i] === listener) {
      listeners.splice(i, 1);
      break;
    }
  }
};

SimileAjax.ListenerQueue.prototype.fire = function(handlerName, args) {
  var listeners = [].concat(this._listeners);
  for (var i = 0; i < listeners.length; i++) {
    var listener = listeners[i];
    if (handlerName in listener) {
      try {
        listener[handlerName].apply(listener, args);
      } catch (e) {
        console.error('Error firing event of name ' + handlerName, e);
        throw e;
      }
    } else if (this._wildcardHandlerName != null &&
            this._wildcardHandlerName in listener) {
      try {
        listener[this._wildcardHandlerName](handlerName);
      } catch (e) {
        console.error('Error firing event of name ' + handlerName + ' to wildcard handler', e);
        throw e;
      }
    }
  }
};

