/*
 * From: https://github.com/simile-widgets/ancient-simile-widgets
 *
 *  (c) Copyright The SIMILE Project 2006. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
import {SimileAjax} from './init.js';

/*==================================================
 *  DOM Utility Functions
 *==================================================
 */

SimileAjax.DOM = {};

SimileAjax.DOM.registerEventWithObject = function(elmt, eventName, obj, handlerName) {
  SimileAjax.DOM.registerEvent(elmt, eventName, function(elmt2, evt, target) {
    return obj[handlerName](elmt2, evt, target);
  });
};

SimileAjax.DOM.registerEvent = function(elmt, eventName, handler) {
  function handler2(evt) {
    if (evt) {
      var target = (evt.target) ? evt.target : ((evt.srcElement) ? evt.srcElement : null);
      if (target) {
        target = (target.nodeType === 1 || target.nodeType === 9) ? target : target.parentNode;
      }

      return handler(elmt, evt, target);
    }
    return true;
  }

  elmt.addEventListener(eventName, handler2, false);
};

SimileAjax.DOM.getPageCoordinates = function(elmt) {
  var left = 0;
  var top = 0;

  if (elmt.nodeType !== 1) {
    elmt = elmt.parentNode;
  }

  var elmt2 = elmt;
  while (elmt2 != null) {
    left += elmt2.offsetLeft;
    top += elmt2.offsetTop;
    elmt2 = elmt2.offsetParent;
  }

  var body = document.body;
  while (elmt != null && elmt !== body) {
    if ('scrollLeft' in elmt) {
      left -= elmt.scrollLeft;
      top -= elmt.scrollTop;
    }
    elmt = elmt.parentNode;
  }

  return {left: left, top: top};
};

SimileAjax.DOM.getEventRelativeCoordinates = function(evt, elmt) {
  var coords = SimileAjax.DOM.getPageCoordinates(elmt);

  return {
    x: evt.pageX - coords.left,
    y: evt.pageY - coords.top
  };
};

SimileAjax.DOM.cancelEvent = function(evt) {
  evt.returnValue = false;
  evt.cancelBubble = true;
  if ('preventDefault' in evt) {
    evt.preventDefault();
  }
};

