/*
 * From: https://github.com/simile-widgets/ancient-simile-widgets
 *
 *  (c) Copyright The SIMILE Project 2006. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
import {SimileAjax} from './init.js';

/*==================================================
 *  Platform Utility Functions and Constants
 *==================================================
 */

/*  This must be called after our jQuery has been loaded
    but before control returns to user-code.
*/


/*==================================================
 *  REMEMBER to update the Version!
 *==================================================
 */
SimileAjax.version = 'pre 2.3.0';

SimileAjax.Platform = {};
SimileAjax.Platform.os = {
  isMac:   false,
  isWin:   false,
  isWin32: false,
  isUnix:  false
};
SimileAjax.Platform.browser = {
  isIE:           false,
  isNetscape:     false,
  isMozilla:      false,
  isFirefox:      false,
  isOpera:        false,
  isSafari:       false,

  majorVersion:   0,
  minorVersion:   0
};

(function() {
  var an = navigator.appName.toLowerCase();
  var ua = navigator.userAgent.toLowerCase();

  /*
     *  Operating system
     */
  SimileAjax.Platform.os.isMac = (ua.indexOf('mac') !== -1);
  SimileAjax.Platform.os.isWin = (ua.indexOf('win') !== -1);
  SimileAjax.Platform.os.isWin32 = SimileAjax.Platform.isWin && (
    ua.indexOf('95') !== -1 ||
        ua.indexOf('98') !== -1 ||
        ua.indexOf('nt') !== -1 ||
        ua.indexOf('win32') !== -1 ||
        ua.indexOf('32bit') !== -1
  );
  SimileAjax.Platform.os.isUnix = (ua.indexOf('x11') !== -1);

  /*
     *  Browser
     */
  SimileAjax.Platform.browser.isIE = (an.indexOf('microsoft') !== -1);
  SimileAjax.Platform.browser.isNetscape = (an.indexOf('netscape') !== -1);
  SimileAjax.Platform.browser.isMozilla = (ua.indexOf('mozilla') !== -1);
  SimileAjax.Platform.browser.isFirefox = (ua.indexOf('firefox') !== -1);
  SimileAjax.Platform.browser.isOpera = (an.indexOf('opera') !== -1);
  SimileAjax.Platform.browser.isSafari = (an.indexOf('safari') !== -1);

  function parseVersionString(s) {
    var a = s.split('.');
    SimileAjax.Platform.browser.majorVersion = parseInt(a[0], 10);
    SimileAjax.Platform.browser.minorVersion = parseInt(a[1], 10);
  }
  function indexOf(s, sub, start) {
    var i = s.indexOf(sub, start);
    return i >= 0 ? i : s.length;
  }

  var offset;
  if (SimileAjax.Platform.browser.isMozilla) {
    offset = ua.indexOf('mozilla/');
    if (offset >= 0) {
      parseVersionString(ua.substring(offset + 8, indexOf(ua, ' ', offset)));
    }
  }
  if (SimileAjax.Platform.browser.isIE) {
    offset = ua.indexOf('msie ');
    if (offset >= 0) {
      parseVersionString(ua.substring(offset + 5, indexOf(ua, ';', offset)));
    }
  }
  if (SimileAjax.Platform.browser.isNetscape) {
    offset = ua.indexOf('rv:');
    if (offset >= 0) {
      parseVersionString(ua.substring(offset + 3, indexOf(ua, ')', offset)));
    }
  }
  if (SimileAjax.Platform.browser.isFirefox) {
    offset = ua.indexOf('firefox/');
    if (offset >= 0) {
      parseVersionString(ua.substring(offset + 8, indexOf(ua, ' ', offset)));
    }
  }
})();
