/*
 * From: https://github.com/simile-widgets/ancient-simile-widgets
 *
 *  (c) Copyright The SIMILE Project 2006. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
import {SimileAjax} from './init.js';

/*==================================================
 *  Default Unit
 *==================================================
 */

SimileAjax.NativeDateUnit = {};

SimileAjax.NativeDateUnit.makeDefaultValue = function() {
  return new Date();
};

SimileAjax.NativeDateUnit.cloneValue = function(v) {
  return new Date(v.getTime());
};

SimileAjax.NativeDateUnit.getParser = function(format) {
  if (typeof format == 'string') {
    format = format.toLowerCase();
  }
  return (format === 'iso8601' || format === 'iso 8601')
        ? SimileAjax.DateTime.parseIso8601DateTime
        : SimileAjax.DateTime.parseGregorianDateTime;
};

SimileAjax.NativeDateUnit.parseFromObject = function(o) {
  return SimileAjax.DateTime.parseGregorianDateTime(o);
};

SimileAjax.NativeDateUnit.toNumber = function(v) {
  return v.getTime();
};

SimileAjax.NativeDateUnit.fromNumber = function(n) {
  return new Date(n);
};

SimileAjax.NativeDateUnit.compare = function(v1, v2) {
  var n1;
  var n2;
  if (typeof v1 == 'object') {
    n1 = v1.getTime();
  } else {
    n1 = Number(v1);
  }
  if (typeof v2 == 'object') {
    n2 = v2.getTime();
  } else {
    n2 = Number(v2);
  }

  return n1 - n2;
};

SimileAjax.NativeDateUnit.earlier = function(v1, v2) {
  return SimileAjax.NativeDateUnit.compare(v1, v2) < 0 ? v1 : v2;
};

SimileAjax.NativeDateUnit.later = function(v1, v2) {
  return SimileAjax.NativeDateUnit.compare(v1, v2) > 0 ? v1 : v2;
};

SimileAjax.NativeDateUnit.change = function(v, n) {
  return new Date(v.getTime() + n);
};

