/*
 * From: https://github.com/simile-widgets/ancient-simile-widgets
 *
 *  (c) Copyright The SIMILE Project 2006. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
import {Timeline} from './init.js';

/*==================================================
 *  Classic Theme
 *==================================================
 */



Timeline.ClassicTheme = new Object();

Timeline.ClassicTheme.create = function() {
  return new Timeline.ClassicTheme._Impl();
};

Timeline.ClassicTheme._Impl = function() {
  this.firstDayOfWeek = 0; // Sunday

  // Note: Many styles previously set here are now set using CSS
  //       The comments indicate settings controlled by CSS, not
  //       lines to be un-commented.
  //
  //
  // Attributes autoWidth, autoWidthAnimationTime, timeline_start
  // and timeline_stop must be set on the first band's theme.
  // The other attributes can be set differently for each
  // band by using different themes for the bands.
  this.autoWidth = false; // Should the Timeline automatically grow itself, as
  // needed when too many events for the available width
  // are painted on the visible part of the Timeline?
  this.autoWidthAnimationTime = 500; // mSec
  this.timeline_start = null; // Setting a date, eg new Date(Date.UTC(2008,0,17,20,00,00,0)) will prevent the
  // Timeline from being moved to anytime before the date.
  this.timeline_stop = null;  // Use for setting a maximum date. The Timeline will not be able
  // to be moved to anytime after this date.
  this.ether = {
    backgroundColors: [
      // "#EEE",
      // "#DDD",
      // "#CCC",
      // "#AAA"
    ],
    highlightOpacity: 50,
    interval: {
      line: {
        show: true,
        opacity: 25
      },
      weekend: {
        opacity: 30
      },
      marker: {
        hAlign: 'Bottom',
        vAlign: 'Right'
      }
    }
  };

  this.event = {
    track: {
      height: 10, // px. You will need to change the track
      //     height if you change the tape height.
      gap:  2, // px. Gap between tracks
      offset:  2, // px. top margin above tapes
      autoWidthMargin:  1.5
      /* autoWidthMargin is only used if autoWidth (see above) is true.
         The autoWidthMargin setting is used to set how close the bottom of the
         lowest track is to the edge of the band's div. The units are total track
         width (tape + label + gap). A min of 0.5 is suggested. Use this setting to
         move the bottom track's tapes above the axis markers, if needed for your
         Timeline.
      */
    },
    overviewTrack: {
      offset: 20, // px -- top margin above tapes
      tickHeight:  6, // px
      height:  2, // px
      gap:  1, // px
      autoWidthMargin:  5 // This attribute is only used if autoWidth (see above) is true.
    },
    tape: {
      height:         4 // px. For thicker tapes, remember to change track height too.
    },
    instant: {
      // default icon. Icon can also be specified per event
      iconWidth: 10,
      iconHeight: 10,
      impreciseOpacity: 20, // opacity of the tape when durationEvent is false
      impreciseIconMargin: 3   // A tape and an icon are painted for imprecise instant
      // events. This attribute is the margin between the
      // bottom of the tape and the top of the icon in that
      // case.
    //        color:             "#58A0DC",
    //        impreciseColor:    "#58A0DC",
    },
    duration: {
      impreciseOpacity: 20 // tape opacity for imprecise part of duration events
      //      color:            "#58A0DC",
      //      impreciseColor:   "#58A0DC",
    },
    label: {
      backgroundOpacity: 50, // only used in detailed painter
      offsetFromLine:  3 // px left margin amount from icon's right edge
      //      backgroundColor:   "white",
      //      lineColor:         "#58A0DC",
    },
    highlightColors: [  // Use with getEventPainter().setHighlightMatcher
      // See webapp/examples/examples.js
      '#FFFF00',
      '#FFC000',
      '#FF0000',
      '#0000FF'
    ],
    highlightLabelBackground: false, // When highlighting an event, also change the event's label background?
  };

  this.mouseWheel = 'scroll'; // 'default', 'zoom', 'scroll'
};
